/*
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.controller;

import java.util.Set;

import com.bbn.morph.executer.Update;
import com.bbn.morph.model.RobotAttributeValues;
import com.bbn.morph.model.PhysicalModel;

/**
 * A controller for that manipulates the {@link PhysicalModel} at an attribute
 * value level.
 * @author aadler
 *
 */
public class ExecuterController {

  private PhysicalModel m_model;
  private PhysicalModel m_environmentModel;
  
  public ExecuterController(PhysicalModel robotModel, PhysicalModel environmentModel) {
    m_model = robotModel;
    m_environmentModel = environmentModel;
  }
  
  /**
   * Perform a set of updates in the form of deltas
   * @param updates updates to perform (deltas)
   */
  public void update(Set<Update> updates) {
    for (Update update : updates) {
      boolean updated = m_model.updateRobotAttributeFromDeltas(
          new RobotAttributeValues(update.getAttribute().getName(), update.getVal()));
      if (!updated)
        updated = m_environmentModel.updateRobotAttributeFromDeltas(
            new RobotAttributeValues(update.getAttribute().getName(), update.getVal()));
      if (!updated)
        System.err.println("Could not find body "+update.getAttribute().getName());
    }
  }

}
