/* Evaluator for robot flipping
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.evaluator;

import java.util.HashSet;
import java.util.Set;

import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.StressInputType;
import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.test.FBTestGenerator;
import com.bbn.morph.view.AttributeValueView;
import com.bbn.morph.view.RobotAttribute;

public class FlippingEvaluator implements Evaluator {

  private AttributeValueView m_model;
  private String m_rFlip;
  private String m_lFlip;
  private String m_body;
  private String m_torque;
  private String m_featureName;
  private String m_stressType;

  @Deprecated
  public FlippingEvaluator(AttributeValueView model) {
    this(model, FBTestGenerator.flippingFeature, StressInputType.FLIPPER_TORQUE_RATIO.toString().toLowerCase(),
        FBTestGenerator.rightflipperLengthAttribute,
        FBTestGenerator.leftflipperLengthAttribute,
        FBTestGenerator.bodyLengthAttribute,
        FBTestGenerator.torqueAttribute);
  }
  
  public FlippingEvaluator(AttributeValueView model, String featureName, 
      String stressType, String rightFlipper,
      String leftFlipper, String body, String torque) {
    m_model = model;
    m_featureName = featureName;
    m_stressType = stressType;
    m_rFlip = rightFlipper;
    m_lFlip = leftFlipper;
    m_body = body;
    m_torque = torque;
  }
  
  @Override
  public Set<StressInputs> getStressInputs() {
	m_model.refresh();  
	Measurement flipperLength = Measurement.max(m_model.get(new RobotAttribute(m_rFlip)),
			  						  m_model.get(new RobotAttribute(m_lFlip)));
	Measurement bodyLength = m_model.get(new RobotAttribute(m_body));
    Measurement torque = m_model.get(new RobotAttribute(m_torque));
    
    double densityTimesG=0.75;
    
    Double ratio = densityTimesG*bodyLength.getStandardValue() / torque.getStandardValue();
    
    StressInputs si = new StressInputs(new Feature(m_featureName));
    si.put(StressInputType.fromString(m_stressType), ratio);
    
    HashSet<StressInputs> results = new HashSet<StressInputs>();
    results.add(si);
    return results;
  }

}
