/* main evaluation controller
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.evaluator;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.model.CompositeModel;
import com.bbn.morph.model.InvalidModelException;
import com.bbn.morph.view.AttributeValueView;
import com.bbn.morph.view.DesiredAttributeValueView;

public class MainEvaluationController implements Evaluator {
  
  List<Evaluator> m_evaluators = new ArrayList<Evaluator>();
  
  public MainEvaluationController(CompositeModel compositeModel, AttributeValueView model, DesiredAttributeValueView dModel) throws InvalidModelException {
    List<Evaluator> evaluators = EvaluatorParser.parseEvaluators(compositeModel.getEvaluatorStream(), compositeModel, model, dModel);
    
    for (Evaluator evaluator : evaluators) {
      m_evaluators.add(evaluator);
    }
  }

  public Set<StressInputs> getStressInputs() {
    Set<StressInputs> inputs = new HashSet<StressInputs>();
    for (Evaluator evaluator : m_evaluators) {
      inputs.addAll(evaluator.getStressInputs());
    }
    return inputs;
  }
}
