/* Motor evaluator
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.evaluator;

import java.util.HashSet;
import java.util.Set;

import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.StressInputType;
import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.UnitPrefixes;
import com.bbn.morph.model.test.FBTestGenerator;
import com.bbn.morph.view.AttributeValueView;
import com.bbn.morph.view.RobotAttribute;

public class MotorComponentViabilityEvaluator implements Evaluator {

  private AttributeValueView m_model;
  private String m_motorMass;
  private String m_torque;
  private String m_featureName;
  private String m_stressType;

  @Deprecated
  public MotorComponentViabilityEvaluator(AttributeValueView model) {
    this(model, FBTestGenerator.MotorViabilityFeature, 
        StressInputType.MOTOR_VIABILITY.toString().toLowerCase(), 
        FBTestGenerator.motorMassAttribute, 
        FBTestGenerator.torqueAttribute);
  }
  
  public MotorComponentViabilityEvaluator(AttributeValueView model,
      String fetureName, String stressType, String motorMass, String torque) {
    m_model = model;
    m_featureName = fetureName;
    m_stressType = stressType;
    m_motorMass = motorMass;
    m_torque = torque;
  }
  
  @Override
  public Set<StressInputs> getStressInputs() {
	m_model.refresh();  
	Measurement mass = m_model.get(new RobotAttribute(m_motorMass));
	Measurement torque = m_model.get(new RobotAttribute(m_torque));
   // assumes the viability is above the line 4.8*torque + 7 
    double boundryMass = 4.8*torque.getStandardValue()+7;
    double ratio = (mass.getValueIn(UnitPrefixes.NONE).getPrefixValue()-boundryMass)/boundryMass;
    
    StressInputs si = new StressInputs(new Feature(m_featureName));
    si.put(StressInputType.fromString(m_stressType), ratio);
    
    HashSet<StressInputs> results = new HashSet<StressInputs>();
    results.add(si);
    return results;
  }

}
