/* Evaluator that ties to Proto
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.evaluator;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashSet;
import java.util.Scanner;
import java.util.Set;

import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.StressInputType;
import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.model.CompositeModel;
import com.bbn.morph.model.test.FBTestGenerator;
import com.bbn.morph.view.ProtoModelView;

@Deprecated
public class ProtoEvaluator implements Evaluator {

  public static final String defaultFileName = "resources/xmlModels/autoGenProtoModel.xml";
  private CompositeModel m_model;
  private String m_featureName;
  private String m_stressType;

  @Deprecated
  public ProtoEvaluator(CompositeModel model) {
    this(model, FBTestGenerator.climbFeature, StressInputType.ANGLE.toString().toLowerCase());
  }


  public ProtoEvaluator(CompositeModel model, String featureName,
      String stressType) {
    m_model = model;
    m_featureName = featureName;
    m_stressType = stressType;
  }

  public Double  attemptProtoCall(String filename) throws Exception {
    ProtoModelView cmv = new ProtoModelView(m_model.getRobotModel(), 
        m_model.getEnvironmentModel());
    File file =new File(filename);
    cmv.writeToFile(file);

    boolean processEndDetected = false;

    Runtime runtime = Runtime.getRuntime();
    // "/Users/aadler/Documents/Projects/MADV/team-repository/code/blueprints/simple-bot.xml"

    int bodies = cmv.getNumberOfBodies();

    String[] cmd = {"/usr/local/bin/proto", "-headless", "-L", "odedynamics", "-body", 
        file.getAbsolutePath(), 
        "-n", Integer.toString(bodies), "-m", "-dim", "10", "10", "-3d", 
        "-floor", "-gravity", "-10","-l",
        "-s", "0.1", "(all (red (bump)) (madvreport (bump) 0))", "-L", "madv",
        "-stop-after", "50", "-T", "--end-x", Double.toString(cmv.getCenterOfStep().getStandardValue())};
    //"--DLL", "libmadvplugin.la", 



    Process exec = runtime.exec(cmd);
    int pingTime = 500;

    // monitor the streams from the process
    TimeExtractionThread inputStreamThread = new TimeExtractionThread(exec.getInputStream(), Thread.currentThread());
    //StreamMonitorThread inputStreamThread = new StreamMonitorThread(exec.getInputStream(), System.out, Thread.currentThread());
    inputStreamThread.start();
    ErrorStreamMonitorThread errorStreamThread = new ErrorStreamMonitorThread(exec.getErrorStream(), System.err, Thread.currentThread());
    errorStreamThread.start();

    while (!processEndDetected) {
      boolean stillRunning = isProcessRunning(exec);
      if (!stillRunning) {
        processEndDetected = true;
        // we monitor the main thread and it has exited
        System.out.println("Process killed.\n");
        inputStreamThread.interrupt();
        errorStreamThread.interrupt();
        exec.destroy();
      }
      else {
        try {
          // the main thread is still running, so wait and then check again
          Thread.sleep(pingTime);
        }
        catch (InterruptedException ie) {
          // here we just want to go and immediately check for a crash
          if (errorStreamThread.isDone() || inputStreamThread.isDone() ){
            inputStreamThread.interrupt();
            errorStreamThread.interrupt();
            exec.destroy();
            processEndDetected=true;
          }

        }
      }
    }
    return inputStreamThread.getMetric();
  }

  /**
   * @param p the {@link Process} to check to see if it is still running
   * @return true iff the process is still running
   */
  private boolean isProcessRunning(Process p) {
    try {
      p.exitValue();
      return false;
    } catch (IllegalThreadStateException e) {
      return true;
    }
  }


  private class ErrorStreamMonitorThread extends Thread {
    private InputStream m_inputStream;
    private OutputStream m_outputStream;
    private Thread m_thread;
    private boolean m_done = false;

    public ErrorStreamMonitorThread(InputStream inputStream, OutputStream outputStream, Thread thread) {
      m_inputStream = inputStream;
      m_outputStream = outputStream;
      m_thread = thread;
    }

    @Override
    public void run() {
      // FIXME this is not efficient
      int b;
      try {
        while ((b = m_inputStream.read()) != -1) {
          m_outputStream.write(b);
        }
      }
      catch (IOException e) {
        System.err.println("Error stream monitor exception"+e.toString());
      }

      // if we run out of input, then we should flush and interrupt the parent 
      // thread
      m_done =true;
      m_thread.interrupt();
    }

    public boolean isDone(){
      return m_done;
    }
  }

  //  private class StreamMonitorThread extends Thread {
  //    private InputStream m_inputStream;
  //    private OutputStream m_outputStream;
  //    private Thread m_thread;
  //
  //    public StreamMonitorThread(InputStream inputStream, OutputStream outputStream, Thread thread) {
  //      m_inputStream = inputStream;
  //      m_outputStream = outputStream;
  //      m_thread = thread;
  //    }
  //    
  //    @Override
  //    public void run() {
  //      // FIXME this is not efficient
  //      int b;
  //      try {
  //        while ((b = m_inputStream.read()) != -1)
  //          m_outputStream.write(b);
  //      }
  //      catch (IOException e) {
  //        System.err.println("Stream monitor exception"+ e.toString());
  //      }
  //      
  //      // if we run out of input, then we should flush and interrupt the parent 
  //      // thread
  //      m_thread.interrupt();
  //    }
  //  }

  private class TimeExtractionThread extends Thread {
    private InputStream m_inputStream;
    private Thread m_thread;
    private Double m_startTime = null;
    private Double m_endTime = null;
    private Double m_climbAngle;
    private boolean m_done =false; 
    public TimeExtractionThread(InputStream inputStream, Thread parent) {
      m_inputStream = inputStream;
      m_thread = parent;
    }

    public Double getMetric() {
      if (m_endTime==null || m_startTime==null)
        return null;
      return m_climbAngle;
      //return m_endTime -  m_startTime;
    }

    @Override
    public void run() {
      // search for the strings we're looking for:

      Scanner scanner = new Scanner(m_inputStream);
      String s = scanner.findWithinHorizon("Started climbing at:", 0);
      System.out.println(s);
      if (scanner.hasNextDouble()){
        m_startTime= new Double(scanner.nextDouble());
        System.out.println(m_startTime);}
      else{
        System.err.println("Expected a double after '"+s +"'");	 
        m_thread.interrupt();
      }

      s = scanner.findWithinHorizon("BodyAngle:", 0);
      System.out.println(s);
      if (scanner.hasNextDouble()){
        m_climbAngle = new Double(scanner.nextDouble());
        System.out.println(m_climbAngle);}
      else{
        System.err.println("Expected a double after '"+s +"'");   
        m_thread.interrupt();
      }

      s = scanner.findWithinHorizon("Finished climbing at:", 0);
      System.out.println(s);
      if (scanner.hasNextDouble()){
        m_endTime = new Double(scanner.nextDouble());
        System.out.println(m_endTime);}
      else{
        System.err.println("Expected a double after '"+s +"'");	 
        m_thread.interrupt();
      }

      //      
      //      // FIXME this is not efficient
      //      int b;
      //      try {
      //        
      //        m_bufferedInputStream.
      //        
      //        while ((b = m_inputStream.read()) != -1)
      //          m_outputStream.write(b);
      //      }
      //      catch (IOException e) {
      //        System.err.println("Stream monitor exception"+ e.toString());
      //      }

      // if we run out of input, then we should flush and interrupt the parent 
      // thread
      m_done= true;
      m_thread.interrupt();
    }

    public boolean isDone(){
      return m_done;
    }
  }

//  public static void main(String[] args) throws Exception {
//    for (int i=0; i<1; i++){
//      ProtoEvaluator p = new ProtoEvaluator(ModelGenerator.getCompositeModel());
//      System.out.println(p.attemptProtoCall(ProtoEvaluator.defaultFileName));}
//  }

  @Override
  public Set<StressInputs> getStressInputs() {
    HashSet<StressInputs> results = new HashSet<StressInputs>();
    Double angle=null;
    try {
      angle = attemptProtoCall(defaultFileName);
    } catch (Exception e) {
      System.err.println("Proto evaluator terminated with exception.");	 
    }
    if (angle==null)
      return results;
    StressInputs si = new StressInputs(new Feature(m_featureName));
    si.put(StressInputType.fromString(m_stressType), angle);
    results.add(si);
    return results;
  }


}
