/* Evaluator for robot properties that compare desired to actual attributes
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.evaluator;

import java.util.HashSet;
import java.util.Set;

import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.StressInputType;
import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.view.AttributeValueView;
import com.bbn.morph.view.DesiredAttributeValueView;
import com.bbn.morph.view.RobotAttribute;

public class RobotDifferenceEvaluator implements Evaluator {

  private AttributeValueView m_model;
  private DesiredAttributeValueView m_desiredModel;
  private String m_stressType;

  @Deprecated
  public RobotDifferenceEvaluator(AttributeValueView model, DesiredAttributeValueView desiredModel) {
    this(model, desiredModel, StressInputType.NORMALIZED_DIFF.toString().toLowerCase());
  }

  public RobotDifferenceEvaluator(AttributeValueView model, DesiredAttributeValueView desiredModel,
      String stressType) {
    m_model = model;
    m_desiredModel = desiredModel;
    m_stressType = stressType;
  }

  
  @Override
  public Set<StressInputs> getStressInputs() {
    Set<StressInputs> inputs = new HashSet<StressInputs>();
   m_model.refresh();
   m_desiredModel.refresh();
   for(RobotAttribute a : m_desiredModel.getAllDesiredAttributes()){
		  Measurement desired = m_desiredModel.get(a);
		  Measurement current = m_model.get(a); 
		 // if (current!= desired){
			  StressInputs in=new StressInputs(genFeature(a.getName()));
			  Measurement value = (desired.subtract(current)).divide(current);
			  in.put(StressInputType.fromString(m_stressType), value.getStandardValue());
			  inputs.add(in);
		 // }
	  }  
    return inputs;
  }

  
/** Generate a FB name using the attribute name and its final value
 * @param a
 * @return
 */
public static Feature genFeature(String a) {
	return new Feature("FB_"+ a);
}

}
