/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.model;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class ComponentLibraries {

  private static final String LIBRARY_PATH = "resources/componentLibraries/";
  
  private Map<String, ComponentLibrary> m_libraries;
  
  public ComponentLibraries() {
    m_libraries = new HashMap<String, ComponentLibrary>();
  }
  
  public void loadLibraries() {
    File libraryDirectory = new File(LIBRARY_PATH);
    if (!libraryDirectory.exists() || !libraryDirectory.isDirectory()) {
      System.err.println("Library directory missing!");
      return;
    }
    
    File[] libraryFiles = libraryDirectory.listFiles(new FilenameFilter() {
      @Override
      public boolean accept(File dir, String name) {
        if (name.endsWith(".csv"))
          return true;
        return false;
      }
    });
    
    for (File libraryFile: libraryFiles) {
      try {
        ComponentLibrary library = new ComponentLibrary(libraryFile);
        m_libraries.put(library.getName(), library);
      }
      catch (IOException e) {
        e.printStackTrace();
      }
    }
  }
  
  /**
   * @param libraryName the library name to look up
   * @return the {@link ComponentLibrary} that the specified name maps to, or null if there is no mapping
   */
  public ComponentLibrary getLibrary(String libraryName) {
    return m_libraries.get(libraryName);
  }
  
}
