/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import com.bbn.morph.evaluator.Evaluator;
import com.bbn.morph.executer.FBModel;
import com.bbn.morph.view.FB;

/**
 * A class to hold all of the models
 * @author aadler
 *
 */
public class CompositeModel {

  private PhysicalModel m_robotModel;
  private DesiredPhysicalModel m_desiredRobotModel;
  private PhysicalModel m_environmentModel;
  private DesiredPhysicalModel m_desiredEnvironmentModel;
  private FBModel m_fbModel;
  private String m_evaluatorsFilename;
  
  public CompositeModel(String robotModelFilename, String environmentModelFilename, ComponentLibraries libraries, String evaluatorsFilename) throws InvalidModelException {
    this(PhysicalModel.loadModel(new File(robotModelFilename), libraries),
        PhysicalModel.loadModel(new File(environmentModelFilename), libraries),
        new FBModel(), evaluatorsFilename);
  }
  
  public CompositeModel(PhysicalModel robotModel, PhysicalModel environmentalModel, FBModel fbModel, String evaluatorsFilename) {
    m_robotModel = robotModel;
    m_environmentModel = environmentalModel;
    m_desiredRobotModel = new DesiredPhysicalModel(m_robotModel);
    m_desiredEnvironmentModel = new DesiredPhysicalModel(m_environmentModel);
    m_fbModel = fbModel;
    m_evaluatorsFilename = evaluatorsFilename;
  }
  
  public void addFunctionalBlueprint(FB fb) {
    m_fbModel.put(fb.getFeature(), fb);
  }
  
  public DesiredPhysicalModel getDesiredRobotModel() {
    return m_desiredRobotModel;
  }
  
  public PhysicalModel getRobotModel() {
    return m_robotModel;
  }
  
  public PhysicalModel getEnvironmentModel() {
    return m_environmentModel;
  }
  
  public DesiredPhysicalModel getDesiredEnvironmentModel() {
    return m_desiredEnvironmentModel;
  }
  
  public FBModel getFbModel() {
    return m_fbModel;
  }
  
  /**
   * @return an {@link InputStream} containing the {@link Evaluator}s or null if an {@link InputStream} cannot be returned.
   */
  public InputStream getEvaluatorStream()  {
    try {
      return new FileInputStream(new File(m_evaluatorsFilename));
    }
    catch (FileNotFoundException ex) {
      // if we can't find the file, then just return null
      return null;
    }
  }

  /**
   * @return the file name containing the {@link Evaluator}s.
   */
  public String getEvaluatorFilename() {
    return m_evaluatorsFilename;
  }
}
