/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.model;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Represents a modification of a particular {@link PhysicalModel}
 * @author aadler
 *
 */
public class DesiredPhysicalModel {

  private PhysicalModel m_baseModel;
  private HashMap<String, RobotAttributeValues> m_desiredChanges;
  
  /**
   * Construct a {@link DesiredPhysicalModel} for the specified base {@link PhysicalModel}
   * @param baseModel the base {@link PhysicalModel} to base the desired changes on
   */
  public DesiredPhysicalModel(PhysicalModel baseModel) {
    m_baseModel = baseModel;
    m_desiredChanges = new HashMap<String, RobotAttributeValues>();
  }
  
  public void addDesiredAttribute(RobotAttributeValues rav) {
    m_desiredChanges.put(rav.getAttribute(), rav.copy());
  }
  
  /**
   * Remove a desired attribute from the model
   * @param rav remove the desired attribute that is linked to the attribute in this {@link RobotAttributeValues}. The value is ignored.
   */
  public void removeDesiredAttribute(RobotAttributeValues rav) {
    m_desiredChanges.remove(rav.getAttribute());
  }
  
  public PhysicalModel getBaseModel() {
    return m_baseModel;
  }
  
  /**
   * @return true iff all of the desired changes have been made
   */
  public boolean areDesiredChangesMade() {
    Map<String, Measurement> baseAttributes = m_baseModel.getAttributeMap();
    for (RobotAttributeValues desiredChange : m_desiredChanges.values()) {
      if (baseAttributes.get(desiredChange.getAttribute()) != desiredChange.getMeasurement())
        return false;
    }
    return true;
  }
  
  public Set<RobotAttributeValues> getDesiredChanges(){
	  return new HashSet<RobotAttributeValues>(m_desiredChanges.values());
  }
}
