/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.model;

public class MeasurementParse {
  private String m_string;
  private int m_position = 0;
  private UnitNames m_unit = null;
  private UnitPrefixes m_prefix = null;
  
  public MeasurementParse(String str, UnitPrefixes expectedPrefix, UnitNames expectedUnit) {
    m_string = str;
    m_prefix = expectedPrefix;
    m_unit = expectedUnit;
  }
  
  public MeasurementParse copy() {
    MeasurementParse copy = new MeasurementParse(getString(), getPrefix(), getUnit());
    copy.increasePosition(getPosition());
    return copy;
  }
  
  public String getString() {
    return m_string;
  }
  
  public int getPosition() {
    return m_position;
  }
  
  public void increasePosition(int amount) {
    m_position += amount;
  }
  
  public UnitPrefixes getPrefix() {
    return m_prefix;
  }
  
  public void setPrefix(UnitPrefixes prefix) {
    m_prefix = prefix;
  }
  
  public UnitNames getUnit() {
    return m_unit;
  }
  
  public void setUnit(UnitNames unit) {
    m_unit = unit;
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + m_position;
    result = prime * result + ((m_prefix == null) ? 0 : m_prefix.hashCode());
    result = prime * result + ((m_string == null) ? 0 : m_string.hashCode());
    result = prime * result + ((m_unit == null) ? 0 : m_unit.hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    MeasurementParse other = (MeasurementParse) obj;
    if (m_position != other.m_position)
      return false;
    if (m_prefix != other.m_prefix)
      return false;
    if (m_string == null) {
      if (other.m_string != null)
        return false;
    } else if (!m_string.equals(other.m_string))
      return false;
    if (m_unit != other.m_unit)
      return false;
    return true;
  }
}