/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.model;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Represents a fixed joint in our model
 * @author aadler
 *
 */
public class ModelFixedJoint extends ModelJoint {

  // FIXME this is fixed in the xml file, but fixedJoint in the xsd
  protected static final String TYPE_NAME = "fixed";
  
  /**
   * Create a fixed joint
   * @param bodyId1 the id for the first body
   * @param bodyId2 the id for the second body
   */
  public ModelFixedJoint(String bodyId1, String bodyId2) {
    super(bodyId1, bodyId2);
  }
  
  @Override
  public ModelObject copy() {
    return new ModelFixedJoint(getBodyId1(), getBodyId2());
  }
  
  /**
   * Creates a {@link ModelFixedJoint} from the given XML element
   * @param element the XML element representing a {@link ModelFixedJoint}
   * @param id1 the first id
   * @param id1 the second id
   * @return the {@link ModelFixedJoint} encoded by the XML
   * @throws InvalidModelException if the XML is ill-formed
   */
  public static ModelJoint createFromXML(Element element, String id1, 
      String id2) {
    // there are not any other parameters so:
    return new ModelFixedJoint(id1, id2);
  }

  @Override
  public Element toXMLElement(Document doc) {
    Element baseElement = super.toXMLElement(doc);
    baseElement.setAttribute("type", TYPE_NAME);
    
    // don't have anything extra to set
    
    return baseElement;
  }

}
