/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.model;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * A top-level abstract class with subclasses {@link ModelBody}, {@link ModelJoint}, and {@link ModelTransform}
 * We link to the parent objects so we can find the right transforms to update.
 * 
 * @author aadler
 *
 */
public abstract class ModelObject {
  
  private ModelObject m_parentObject = null;
  
  /**
   * @return the parent object of this {@link ModelObject}
   */
  public ModelObject getParentObject() {
    return m_parentObject;
  }
  
  /**
   * @param parentObject the {@link ModelObject} that is the parent of this {@link ModelObject}
   */
  public void setParentObject(ModelObject parentObject) {
    m_parentObject = parentObject;
  }

  /**
   * @param doc the XML document
   * @return the {@link ModelObject} as an XML {@link Element} in the {@link Document}
   */
  public abstract Element toXMLElement(Document doc);
  
  /**
   * @return a copy of the {@link ModelObject}. The copy is non-recursive and does not copy or link to the {@link ModelObject}s parent object (if any).
   */
  public abstract ModelObject copy();

}
