/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.model;

import java.util.Collection;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Represents an abstract rotation in our model
 * @author aadler
 *
 */
public abstract class ModelRotation {

  /**
   * @param prefix the prefix to use for the attributes
   * @return a list of attributes representing the {@link ModelRotation}
   */
  public abstract Collection<RobotAttributeValues> getAttributes(String prefix);

  /**
   * @param attribute the attribute to check for handling
   * @return true iff the specified attribute is handled by the {@link ModelRotation}
   */
  public abstract boolean isHandledAttribute(String attribute);

  /**
   * @param attribute the attribute to get
   * @return the value of the specified attribute (or null if not handled)
   */
  public abstract Measurement getAttributeValue(String attribute);

  /**
   * Update the value of the specified attribute
   * @param attribute the atttribute to update
   * @param delta the delta to change the attribute by
   */
  public abstract void updateAttributeByDelta(String attribute, double delta);

  /**
   * @param doc the XML document
   * @return the {@link ModelRotation} as an XML {@link Element} in the {@link Document}
   */
  public abstract Element toXMLElement(Document doc);
  
  /**
   * Creates a {@link ModelRotation} from the given XML element
   * @param element the XML element representing a {@link ModelRotation}
   * @return the {@link ModelRotation} encoded by the XML
   * @throws InvalidModelException if the XML is ill-formed
   */
  public static ModelRotation createFromXML(Element element) throws InvalidModelException {
    String type = element.getTagName();
    
    if (type.equals(ModelQuaternion.TYPE_NAME)) {
      return ModelQuaternion.createFromXML(element);
    }
    else 
      throw new RuntimeException("Unhandled type: "+type);
  }

  /**
   * @return a copy of the {@link ModelRotation}.
   */
  public abstract ModelRotation copy();
  
}
