/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.model;

public enum UnitPrefixes {
	
	NONE("", "", 1),
	KILO("kilo", "k", 1000),
	HEXA("hexa", "h", 100),
	DECA("deca", "dk", 10),
	DECI("deci", "d", .1),
	CENTI("centi", "c", .01),
	MILLI("milli", "m", .001);
	
	private String m_name;
	private String m_symbol;
	private double m_convert;
	
	UnitPrefixes (String name, String symbol, double convert){
		m_name = name;
		m_symbol = symbol;
		m_convert = convert;
	}
	
	UnitPrefixes (UnitPrefixes tocopy){
		m_name = tocopy.getName();
		m_symbol = tocopy.getSymbol();
		m_convert = tocopy.getConvert();
	}
	
	/**
	 * @return returns the name of the Unit's prefix
	 */
	public String getName() {
		return m_name;
	}
	
	/**
	 * @return returns the symbol associated with the Unit's prefix (eg. millimeters = "m")
	 */
	public String getSymbol() {
		return m_symbol;
	}
	
	/**
	 * @return returns a factor by which a measurement in the given prefix can be converted through multiplication back to standard MGS units.
	 */
	public double getConvert() {
		return m_convert;
	}
	
	@Override
	public String toString() {
		return getName();
	}
	
}
