/* Create functional blueprints for testing purposes.
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.model.test;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.Pair;
import com.bbn.morph.executer.StressInputType;
import com.bbn.morph.view.FB;
import com.bbn.morph.view.RobotAttribute;
import com.bbn.morph.view.StepDecrementUpdate;
import com.bbn.morph.view.StepIncrementUpdate;
import com.bbn.morph.view.StressFunction;
import com.bbn.morph.view.SymmetricFivePieceLinearStressFunction;
import com.bbn.morph.view.ThreePieceLinearStressFunction;
import com.bbn.morph.view.UpdateFunction;

/**
 * Generate some FBs to do some testing -- eventually it should be moved to files...
 * @author aadler
 *
 */
public class FBTestGenerator {
  final public static String rightflipperLengthAttribute = "rightarm.lengthz";
  final public static String leftflipperLengthAttribute = "leftarm.lengthz";
  final public static String bodyLengthAttribute = "botbody.lengthx";
  final public static String torqueAttribute = "motor.torque";
  final public static String motorMassAttribute = "motor.mass";

  final public static String climbFeature = "Climb FB";
  final public static String boxFeature = "Proportion FB";
  final public static String flippingFeature = "Flipping FB";
  final public static String MaxTorqueFeature = "MaxTorque FB";
  final public static String MotorMassFeature = "MotorMass FB";
  final public static String MotorViabilityFeature="MotorViability FB";

  public static List<FB> getFBs() {
    ArrayList<FB> fbList = new ArrayList<FB>();
    fbList.add(createFBfeatureProto());
    fbList.add(createFBfeature2());
 //   fbList.add(createFBFlipperTorque());
 //   fbList.add(createFBMaxTorque());
 //   fbList.add(createFBMotorMass());
 //   fbList.add(createFBMotorViability());
    
    return fbList;
  }
  
  // FB for climbing
  private static FB createFBfeature1(){
    Pair first = new Pair(3000,0);
    Pair second = new Pair(5000,10000);
    StressInputType inputLabel = StressInputType.TIME;
    StressFunction stressF = new ThreePieceLinearStressFunction(inputLabel, first, second);
    UpdateFunction updateF1 = new StepDecrementUpdate(2);
    UpdateFunction updateF2 = new StepDecrementUpdate(1);
    HashMap<RobotAttribute, UpdateFunction> updateFunctions = new HashMap<RobotAttribute, UpdateFunction>();
    updateFunctions.put(new RobotAttribute(rightflipperLengthAttribute), updateF1);
    updateFunctions.put(new RobotAttribute(leftflipperLengthAttribute), updateF1);
    updateFunctions.put(new RobotAttribute(bodyLengthAttribute), updateF2);
    FB testFB = new FB(new Feature(climbFeature), updateFunctions, stressF);
    return testFB;
  }

  // FB for boxability
  private static FB createFBfeature2(){
    Pair first = new Pair(1.05,0);
    Pair second = new Pair(1.5,20);
    StressInputType inputLabel = StressInputType.FLIPPER_BODY_RATIO;
    StressFunction stressF = new ThreePieceLinearStressFunction(inputLabel, first, second);
    UpdateFunction updateF1 = new StepIncrementUpdate(2);
    HashMap<RobotAttribute, UpdateFunction> updateFunctions = new HashMap<RobotAttribute, UpdateFunction>();
    updateFunctions.put(new RobotAttribute(rightflipperLengthAttribute), updateF1);
    updateFunctions.put(new RobotAttribute(leftflipperLengthAttribute), updateF1);
    FB testFB = new FB(new Feature(boxFeature), updateFunctions, stressF);
    return testFB;
  }

  // FB for proto
  private static FB createFBfeatureProto(){
    Pair first = new Pair(-0.1,1);
    Pair second = new Pair(0.1,0);
    StressInputType inputLabel = StressInputType.ANGLE;
    StressFunction stressF = new ThreePieceLinearStressFunction(inputLabel, first, second);
    UpdateFunction updateF1 = new StepIncrementUpdate(2);
    UpdateFunction updateF2 = new StepIncrementUpdate(1);
    HashMap<RobotAttribute, UpdateFunction> updateFunctions = new HashMap<RobotAttribute, UpdateFunction>();
    updateFunctions.put(new RobotAttribute(rightflipperLengthAttribute), updateF1);
    updateFunctions.put(new RobotAttribute(leftflipperLengthAttribute), updateF1);
    updateFunctions.put(new RobotAttribute(bodyLengthAttribute), updateF2);
    FB testFB = new FB(new Feature(climbFeature), updateFunctions, stressF);
    return testFB;
  }

  
  // FB for FlipperTorque -- flippingFeature
  private static FB createFBFlipperTorque(){
    Pair first = new Pair(1,0);
    Pair second = new Pair(1.05,20);
    StressInputType inputLabel = StressInputType.FLIPPER_TORQUE_RATIO;
    StressFunction stressF = new ThreePieceLinearStressFunction(inputLabel, first, second);
    UpdateFunction updateF1 = new StepDecrementUpdate(0.6);
    HashMap<RobotAttribute, UpdateFunction> updateFunctions = new HashMap<RobotAttribute, UpdateFunction>();
    updateFunctions.put(new RobotAttribute(torqueAttribute), updateF1);
    FB testFB = new FB(new Feature(flippingFeature), updateFunctions, stressF);
    return testFB;
  }
  
  // FB for maxTorque
  private static FB createFBMaxTorque(){
    Pair first = new Pair(11,0);
    Pair second = new Pair(13,20);
    StressInputType inputLabel = StressInputType.TORQUE;
    StressFunction stressF = new ThreePieceLinearStressFunction(inputLabel, first, second);
    UpdateFunction updateF1 = new StepIncrementUpdate(1);
    HashMap<RobotAttribute, UpdateFunction> updateFunctions = new HashMap<RobotAttribute, UpdateFunction>();
    updateFunctions.put(new RobotAttribute(torqueAttribute), updateF1);
    FB testFB = new FB(new Feature(MaxTorqueFeature), updateFunctions, stressF);
    return testFB;
  }
 
  // FB for motor mass 
  private static FB createFBMotorMass(){
    StressInputType inputLabel = StressInputType.MOTOR_MASS;
    double mid= 39.5;
    double innerRadius =29.5;
    double outerRadius= 31.5;
	StressFunction stressF = new SymmetricFivePieceLinearStressFunction(inputLabel, mid, innerRadius, outerRadius);
    UpdateFunction updateF1 = new StepIncrementUpdate(1);
    HashMap<RobotAttribute, UpdateFunction> updateFunctions = new HashMap<RobotAttribute, UpdateFunction>();
    updateFunctions.put(new RobotAttribute(motorMassAttribute), updateF1);
    FB testFB = new FB(new Feature(MotorMassFeature), updateFunctions, stressF);
    return testFB;
  } 
  
  // FB for motor viability 
  private static FB createFBMotorViability(){
    Pair first = new Pair(0,20);
    Pair second = new Pair(0.05,0);
    StressInputType inputLabel = StressInputType.MOTOR_VIABILITY;
    StressFunction stressF = new ThreePieceLinearStressFunction(inputLabel, first, second);
    UpdateFunction updateF1 = new StepIncrementUpdate(1);
    UpdateFunction updateF2 = new StepDecrementUpdate(0.2);
    HashMap<RobotAttribute, UpdateFunction> updateFunctions = new HashMap<RobotAttribute, UpdateFunction>();
    updateFunctions.put(new RobotAttribute(motorMassAttribute), updateF1);
    updateFunctions.put(new RobotAttribute(torqueAttribute), updateF2);
    FB testFB = new FB(new Feature(MotorViabilityFeature), updateFunctions, stressF);
    return testFB;
  }   
  
}
