package com.bbn.morph.model.test; 

import java.io.*;
import java.util.*;

import org.jgap.Chromosome;
import org.jgap.Configuration;
import org.jgap.DefaultFitnessEvaluator;
import org.jgap.FitnessFunction;
import org.jgap.Gene;
import org.jgap.Genotype;
import org.jgap.IChromosome;
import org.jgap.InvalidConfigurationException;
import org.jgap.RandomGenerator;
import org.jgap.event.EventManager;
import org.jgap.impl.*;

import com.bbn.morph.model.InvalidModelException;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.UnitPrefixes;
import com.bbn.morph.model.Units;
import com.bbn.morph.controller.test.ControllerForTesting;;

public class GAFitnessFunction extends FitnessFunction {
	class SmallMutationOperator extends MutationOperator {

		  public SmallMutationOperator()
		      throws InvalidConfigurationException {
			  super();
		  }
		  public SmallMutationOperator(final Configuration a_config, final int a_desiredMutationRate)
                		  throws InvalidConfigurationException {
			  super(a_config, a_desiredMutationRate);
		  }

		  private void mutateGene(final Gene a_gene, final RandomGenerator a_generator) {
			    for (int k = 0; k < a_gene.size(); k++) {
			      // Retrieve value between 0 and 1 (not included) from generator.
			      // Then map this value to range -0.1 and 0.1
			      // -------------------------------------------------------------
			      //double fraction = -1 + a_generator.nextDouble() * GAFitnessFunction.MAX_MUTATION_FRACTION * 2;
			    	double fraction = a_generator.nextDouble() * GAFitnessFunction.MAX_MUTATION_FRACTION * 2 - GAFitnessFunction.MAX_MUTATION_FRACTION;
			      a_gene.applyMutation(k, fraction);
			    }
			  }
	}
	
	static RandomFBModelCreator model;
	public static int changedAttr;
	public static double finalVal;
	public static ArrayList<Double> initSol = new ArrayList<Double>();;
	public static int MAX_EVOLUTIONS = 1000;
	public static int POP_SIZE;
	public static int INV_MUTATION_RATE;
	public static double MAX_MUTATION_FRACTION ;
	static double LARGE_D = 10000;
	public static double MAX_ATTR_VAL; 
	public static boolean USE_STRESS;
	static double STRESS_THRESHOLD = 0.002;
	
	// These weights specify how much we care about
	// 1. wtSimilarityToInit: Having a final solution close to the initial
	// 2. wtDesiredValReached : How closely the desired change was achieved
	// 3. wtConstraintsSat: How much the constraints are satisfied
	public static double wtSimilarityToInit = 1;
	static double wtDesiredValReached = 50;
	static double wtConstraintsSat = 50;
	ControllerForTesting controller;
	
	public GAFitnessFunction(ControllerForTesting c) {
		controller = c;
	}	
	
	public static void extractInitValues(RandomFBModelCreator rfbc) {
		// construct the initial solution from the  model
		model = rfbc;
		initSol.clear();
		for (int a=0; a < model.getModel().getAttributeMap().size(); a++)
			initSol.add(model.model.get(RandomFBModelCreator.attributePrefix+a).getStandardValue());

		changedAttr = (int)Math.ceil(initSol.size()/2);
	}
	
	public static void writeInfoFile(String filePrefix) {
		try {
			PrintWriter out = new PrintWriter(new FileWriter(filePrefix + "/info"));
			out.println("attributes " + initSol + "\nconstraints " + model.constraints + "\nratios " + model.constraintRatios + 
					"\nchange attr " + changedAttr + " to " + finalVal);
			for (int c=0; c < model.constraints.size(); c++){
				int constraint = model.constraints.get(c);
				String[] atts = model.resolveConstraint(constraint);
				out.println("Constraint " + atts[0]+"/"+ atts[1] +" <= " + model.constraintRatios.get(c));

			}				 
			out.close();
			out.println("Popsize " + POP_SIZE);
			out.println("Evolutiosn " + MAX_EVOLUTIONS);
			out.println("InverseMutationRate " + INV_MUTATION_RATE);
			out.println("wtSimilarityToInit " + wtSimilarityToInit);
			out.println("wtDesiredValReached " + wtDesiredValReached);
			out.println("wtConstraintsSat " + wtConstraintsSat);
		} catch (IOException e){
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	public static void main(String[] args) throws InvalidModelException, IOException, InvalidConfigurationException {
		
	}
	
	public Configuration createConfiguration(String name) throws InvalidConfigurationException {
		// This is copied from DefaultConfiguration.
		// -----------------------------------------
		Configuration config = new Configuration(name, "");
		BestChromosomesSelector bestChromsSelector =
				new BestChromosomesSelector(config, 1.0d);
		bestChromsSelector.setDoubletteChromosomesAllowed(false);
		config.addNaturalSelector(bestChromsSelector, true);
		config.setRandomGenerator(new StockRandomGenerator());
		config.setMinimumPopSizePercent(0);
		config.setEventManager(new EventManager());
		config.setFitnessEvaluator(new DefaultFitnessEvaluator());
		config.setChromosomePool(new ChromosomePool());
		config.addGeneticOperator(new CrossoverOperator(config, 0.35d));
		// These are different:
			// --------------------
		config.addGeneticOperator(new SmallMutationOperator(config, INV_MUTATION_RATE));
		//config.addGeneticOperator(new MutationOperator(config, INV_MUTATION_RATE));
		return config;
	}
	
	public void runFromModel(String filePrefix) throws IOException  {
		PrintWriter out = null;
		 try {
			 out = new PrintWriter(new FileWriter(filePrefix));
		 } catch (IOException e){
			  e.printStackTrace();
			  System.exit(1);
		 }
		 
		System.out.println(filePrefix + " atts " + initSol + " cons " + model.constraints);
		// Setting up a GA configuration and a population
		Configuration conf = null;
		IChromosome bestSolutionSoFar = null;
		Genotype population = null;

		try {
			//conf = new DefaultConfiguration(filePrefix, "noname");
			conf = createConfiguration(filePrefix);
			conf.setFitnessFunction(this);	
			conf.setPreservFittestIndividual(true);
			conf.setMinimumPopSizePercent(100);

			Gene[] sampleGenes = new Gene[initSol.size()];
			for (int i=0; i<initSol.size(); i++) {
				sampleGenes[i] = new DoubleGene(conf, 0, MAX_ATTR_VAL );
				//sampleGenes[i].setAllele(initSol.get(i));
			}

			Chromosome sampleChromosome = new Chromosome(conf, sampleGenes );
			conf.setSampleChromosome( sampleChromosome );
			conf.setPopulationSize( POP_SIZE );
			population = Genotype.randomInitialGenotype( conf );
		} catch (InvalidConfigurationException e) {
			System.out.println("genetic algorithm invalid config");
			System.exit(1);
		}

		double bestVal = Double.MAX_VALUE;
		long beforeT =  System.currentTimeMillis();
		int i = 0;
		for(; i < MAX_EVOLUTIONS; i++ ) {
			population.evolve();
			IChromosome currentBest = population.getFittestChromosome();
			double curStress = getStress(currentBest);
			out.println("evo " + i + " " + toString(currentBest) + " " + curStress); // + " " + curBadness  + " sz " + population.getPopulation().size() 

			// Regardless of whether you're using stress to guide GA, keep individual w/ lowest stress
			if (curStress < bestVal) {
				bestSolutionSoFar = currentBest;
				bestVal = curStress;
			}
			
			/*
			if (!USE_STRESS) {
				double curBadness = calcBadness(currentBest, null);
				 if (curBadness < bestVal) {
					 bestSolutionSoFar = currentBest;
					 bestVal = curBadness;
				 }
			}
			*/
			if (curStress <= STRESS_THRESHOLD)
				break;
		}
		long afterT = System.currentTimeMillis();
		out.println("Time " + (afterT - beforeT)/1000.0);
		
		double badness = calcBadness(bestSolutionSoFar, out);
		double stress = getStress(bestSolutionSoFar);
		out.println("stats " + toString(bestSolutionSoFar) + " " + badness + " " + stress);
		// if you never reached desired stress, complain
		if (stress > STRESS_THRESHOLD)
			out.println("Iters -1" );
		else 
			out.println("Iters " + i);
			
		out.close();
	}
	
	double getStress(IChromosome currentBest) {
		// extract values
		double[] newVals = new double[currentBest.size()];
		for (int a=0; a < currentBest.size(); a++) 
			newVals[a] = (Double)currentBest.getGene(a).getAllele();		
		return controller.setAttribsAndEvalStress(newVals);
	}
	
	static String toString(IChromosome chromo) {
		String str = " ";
		for (int a=0; a < chromo.size(); a++) 
			str += chromo.getGene(a).getAllele() + " ";
		return str;
	}

	double calcBadness(IChromosome chromo, PrintWriter out) {
		if (chromo == null)
			return LARGE_D - 1;
		double distFromDesired = 0;
		double sumOfViolations = 0;
		double distFromInit = 0;
		
		// for each attr, sum distance from init solution
		for (int a=0; a < chromo.size(); a++) {
			double curVal = (Double)chromo.getGene(a).getAllele();
			// if this is the attr we wanted to change, penalize by how far we are from desired val
			if (changedAttr == a)
				distFromDesired = Math.abs(finalVal - curVal);
			else {
				// we prefer little change in the other attributes 
				double initVal = initSol.get(a);
				distFromInit += Math.abs(initVal - curVal);				
			}
		}
		
		// for each constraint, see how much it's satisfied
		for (int c=0; c < model.constraints.size(); c++){
			int constraint = model.constraints.get(c);
			int[] atts = model.resolveConstraintInt(constraint);
			double curRatio = (Double)chromo.getGene(atts[0]).getAllele() / (Double)chromo.getGene(atts[1]).getAllele();
			double ratioDiff = curRatio - model.constraintRatios.get(c);
			if (ratioDiff > 0.05) {
				double normalizedViol = ratioDiff / model.constraintRatios.get(c);
				//sumOfViolations += ratioDiff;
				sumOfViolations += normalizedViol;
			}
		}
		
		if (out != null) {
			out.println("attributes " + initSol + "\nconstraints " + model.constraints + "\nratios " + model.constraintRatios + 
					"\nchange attr " + changedAttr + " to " + finalVal);
			out.println(" similarity " + distFromInit + " desired " + distFromDesired + " viols " + sumOfViolations);
		}
		return (wtSimilarityToInit * distFromInit + 
				wtDesiredValReached * distFromDesired + 
				wtConstraintsSat * sumOfViolations);
	}

	@Override
	protected double evaluate(IChromosome chromo) {
		double fitness = (USE_STRESS ? 1/getStress(chromo) : LARGE_D -calcBadness(chromo, null));
		return (fitness < 0.01) ? 0.01 : fitness;
	}

}
