/* Handles 3D mouse navigation in OpenGL
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

/*  This file is adapted from OpenTissue, and has been altered:                                                                                                                  

Copyright (c) 2001-2007 Kenny Erleben http://www.opentissue.org                                                                                                                  

This software is provided 'as-is', without any express or implied                                                                                                                
warranty. In no event will the authors be held liable for any damages                                                                                                            
arising from the use of this software.                                                                                                                                           

Permission is granted to anyone to use this software for any purpose,                                                                                                            
including commercial applications, and to alter it and redistribute it                                                                                                           
freely, subject to the following restrictions:                                                                                                                                   

    1. The origin of this software must not be misrepresented; you must not                                                                                                      
    claim that you wrote the original software. If you use this software                                                                                                         
    in a product, an acknowledgment in the product documentation would be                                                                                                        
    appreciated but is not required.                                                                                                                                             

    2. Altered source versions must be plainly marked as such, and must not be                                                                                                   
    misrepresented as being the original software.                                                                                                                               

    3. This notice may not be removed or altered from any source                                                                                                                 
    distribution.                                                                                                                                                                

 */

// TODO we should clean up this code some -- a bunch of it isn't used, we should also clean up the attribution statement

package com.bbn.morph.ui;

import java.awt.event.MouseEvent;

import javax.swing.SwingUtilities;


public class MouseTracker {
  private double[] m_axis = new double[3];
  private double m_radius = 0.8;
  private double m_angle = 0;
  private double[] m_anchor_position = new double[3];
  private double[] m_current_position = new double[3];
  private double[][] m_xform_anchor = new double[4][4];
  private double[][] m_xform_incremental = new double[4][4];
  private double[][] m_xform_current = new double[4][4];
  //private double[] m_gl_xform_current = new double[16];

  private double[] m_center = {0, 0, 0};
  private double m_zoomFactor = 1;
  
  enum State {NONE, ZOOM, ROTATE};
  
  private double[] m_lastPoint = {0.0, 0.0, 0.0};
  private State m_movement = State.NONE;
  
  //private static double m_ROTSCALE = 90.0;
  private static double m_ZOOMSCALE = 0.008;

  private double m_viewWidth, m_viewHeight;

  private double[] normalizeCoordinates(double x, double y) {
    x = 2.0 * x / m_viewWidth  - 1.0;
    if (x < -1.0) x = -1.0;
    if (x >  1.0) x =  1.0;

    y = -(2.0 * y / m_viewHeight - 1.0);
    if (y < -1.0) y = -1.0;
    if (y >  1.0) y =  1.0;
    
    return new double[]{x, y};
  }

  public double getZoomFactor() {
    return m_zoomFactor;
  }
  
  public double[] getCenter() {
    return m_center;
  }
  
  private static void toIdentity(double[][] matrix) {
    for (int i = 0; i < matrix.length; i++) 
      for (int j = 0; j < matrix[i].length; j++) 
        matrix[i][j] = (i==j)?1:0;
  }

  private void genericReset()
  {
    for(int i=0;i<3;i++) {
      m_anchor_position[i]=0;
      m_current_position[i]=0;
      m_axis[i]=0;
    }
    m_angle = 0.0;
    toIdentity(m_xform_anchor);
    toIdentity(m_xform_incremental);
    toIdentity(m_xform_current);
  }

  private void matMult(double[][] a, double[][] b, double[][] out) {
    for(int i=0;i<4;i++)
      for(int j=0;j<4;j++) {
        double sum = 0;
        for(int k=0;k<4;k++) sum += a[i][k]*b[k][j];
        out[i][j]=sum;
      }
  }

  public double[][] getCurrentRotation()
  {
    // Compute the the rotation from Panchor to Pcurrent, i.e.                                                                                                                   
    // the rotation form (Xanchor, Yanchor, Zanchor) to                                                                                                                          
    // (Xcurrent, Ycurrent, Zcurrent) along a great circle.                                                                                                                      
    // Multiply the IncrementalTransformation and the AnchorTransformation                                                                                                       
    // to get the CurrentTransformation.                                                                                                                                         
    matMult(m_xform_incremental,m_xform_anchor,m_xform_current);
    //System.out.println("rotation: "); print(m_xform_current);
    return m_xform_current;
  }

//  private double[] getGLCurrentRotation()
//  {
//    // Compute the the rotation from Panchor to Pcurrent, i.e.                                                                                                                   
//    // the rotation form (Xanchor, Yanchor, Zanchor) to                                                                                                                          
//    // (Xcurrent, Ycurrent, Zcurrent) along a great circle.                                                                                                                      
//    // Multiply the IncrementalTransformation and the AnchorTransformation                                                                                                       
//    // to get the CurrentTransformation.                                                                                                                                         
//    matMult(m_xform_incremental,m_xform_anchor,m_xform_current);
//
//    m_gl_xform_current[ 0 ] = m_xform_current[ 0 ][ 0 ];
//    m_gl_xform_current[ 1 ] = m_xform_current[ 1 ][ 0 ];
//    m_gl_xform_current[ 2 ] = m_xform_current[ 2 ][ 0 ];
//    m_gl_xform_current[ 3 ] = 0;
//    m_gl_xform_current[ 4 ] = m_xform_current[ 0 ][ 1 ];
//    m_gl_xform_current[ 5 ] = m_xform_current[ 1 ][ 1 ];
//    m_gl_xform_current[ 6 ] = m_xform_current[ 2 ][ 1 ];
//    m_gl_xform_current[ 7 ] = 0;
//    m_gl_xform_current[ 8 ] = m_xform_current[ 0 ][ 2 ];
//    m_gl_xform_current[ 9 ] = m_xform_current[ 1 ][ 2 ];
//    m_gl_xform_current[ 10 ] = m_xform_current[ 2 ][ 2 ];
//    m_gl_xform_current[ 11 ] = 0;
//    m_gl_xform_current[ 12 ] = 0;
//    m_gl_xform_current[ 13 ] = 0;
//    m_gl_xform_current[ 14 ] = 0;
//    m_gl_xform_current[ 15 ] = 1;
//
//    return m_gl_xform_current;
//  }
//
//  private double[][] getIncrementalRotation() { 
//    return m_xform_incremental;
//  }


  public void reset() {
    genericReset();
    projectOntoSurface(m_anchor_position);
    projectOntoSurface(m_current_position);
  }

  private void beginDrag(double x, double y) {
    m_angle = 0.0;
    for(int i=0;i<3;i++) m_axis[i]=0;

    for(int i=0;i<4;i++)
      for(int j=0;j<4;j++)
        m_xform_anchor[i][j] = m_xform_current[i][j];
    toIdentity(m_xform_incremental);
    toIdentity(m_xform_current);

    m_current_position[0] = m_anchor_position[0] = x;
    m_current_position[1] = m_anchor_position[1] = y;
    m_current_position[2] = m_anchor_position[2] = 0;
    projectOntoSurface(m_anchor_position);
    projectOntoSurface(m_current_position);
  }

  private void drag(double x, double y) {
    m_current_position[0] = x;
    m_current_position[1] = y;
    m_current_position[2] = 0;
    projectOntoSurface(m_current_position);
    computeIncremental(m_anchor_position,m_current_position,m_xform_incremental);
  }

  private void endDrag(double x, double y) {
    m_current_position[0] = x;
    m_current_position[1] = y;
    m_current_position[2] = 0;
    projectOntoSurface(m_current_position);
    computeIncremental(m_anchor_position,m_current_position,m_xform_incremental);
  }

  private double length(double[] v) {
    return Math.sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);
  }

  private void toUnit(double[] v) {
    double len = length(v);
    if(len==0) return;
    for(int i=0;i<3;i++) { v[i] /= len; }
  }

  private void projectOntoSurface(double[] p) {
    double radius2 = m_radius * m_radius;
    double length2 = p[0]*p[0] + p[1]*p[1];
    if (length2 <= radius2 / 2.0) {
      p[2] = Math.sqrt(radius2 - length2);
    } else {
      p[2] = radius2 / (2.0 * Math.sqrt(length2));
      double length = Math.sqrt(length2 + p[2] * p[2]);
      for(int i=0;i<3;i++) p[i] /= length;
    }
    toUnit(p);
  }

  private double dot(double[] a, double[] b) {
    return a[0]*b[0] + a[1]*b[1] + a[2]*b[2];
  }

  private void Ru(double radians, double[] axis, double[][] out) {
    double cosinus = Math.cos(radians);
    double sinus   = Math.sin(radians);
    double[] u = { axis[0],axis[1],axis[2] };
    toUnit(u);

    //Foley p.227 (5.76)
    toIdentity(out);
    out[0][0] = u[0]*u[0] + cosinus*(1.0 - u[0]*u[0]);
    out[0][1] = u[0]*u[1]*(1.0-cosinus) - sinus*u[2];
    out[0][2] = u[0]*u[2]*(1.0-cosinus) + sinus*u[1];
    out[1][0] = u[0]*u[1]*(1.0-cosinus) + sinus*u[2];
    out[1][1] = u[1]*u[1] + cosinus*(1.0 - u[1]*u[1]);
    out[1][2] = u[1]*u[2]*(1.0-cosinus) - sinus*u[0];
    out[2][0] = u[0]*u[2]*(1.0-cosinus) - sinus*u[1];
    out[2][1] = u[1]*u[2]*(1.0-cosinus) + sinus*u[0];
    out[2][2] = u[2]*u[2] + cosinus*(1.0 - u[2]*u[2]);
  }

  private void cross(double[] a, double[] b, double[] out) {
    out[0] = a[1]*b[2]-b[1]*a[2];
    out[1] = b[0]*a[2]-a[0]*b[2];
    out[2] = a[0]*b[1]-b[0]*a[1];
  }

  private void computeIncremental(double[] anchor, double[] current, double[][] transform)
  {
    //System.out.println("a "+Arrays.toString(anchor)+ " current "+Arrays.toString(current));
    //print(transform);
    
    cross(anchor, current, m_axis);
    double len = length(m_axis);
    toUnit(m_axis);
    m_angle = Math.atan2(len, dot(anchor,current) );
    //System.out.println("a "+m_angle+ " axis "+Arrays.toString(m_axis));
    
    
    Ru(m_angle, m_axis, transform);
  }


//  public static void print(double[][] foo) {
//    for (int i = 0; i < foo.length; i++) 
//      for (int j = 0; j < foo[i].length; j++) 
//        System.out.print(foo[i][j]+ " ");
//    System.out.println();
//  }
//
//  public static void main(String[] args) {
//    double[][] m_xform_anchor = new double[4][4];
//    print(m_xform_anchor);
//    toIdentity(m_xform_anchor);
//
//    print(m_xform_anchor);
//
//  }

  
  
  public void buttonDown(MouseEvent me) {
    if (SwingUtilities.isLeftMouseButton(me)) {
      double[] normed = normalizeCoordinates(me.getX(), me.getY());
      m_movement = State.ROTATE;
      beginDrag(normed[0], normed[1]);
    }
    else if (SwingUtilities.isRightMouseButton(me)) {
      m_movement = State.ZOOM;
      m_lastPoint = new double[]{me.getX(), me.getY(), 0.0};
    }
  }
  
  
  
  public void buttonUp(MouseEvent me) {
    if (SwingUtilities.isLeftMouseButton(me)) {
      double[] normed = normalizeCoordinates(me.getX(), me.getY());
      endDrag(normed[0], normed[1]);
      m_movement = State.NONE;
    }
    else if (SwingUtilities.isRightMouseButton(me)) {
      m_movement = State.NONE;
    }
    
  }

  public void drag(MouseEvent me) {
    double pixelDiff;
    double[] curPoint = new double[3];

    switch(m_movement) {
    case ROTATE:
      // Left-mouse button is being held down 
      double[] normed = normalizeCoordinates(me.getX(), me.getY());
      drag(normed[0], normed[1]);
      break;
    case ZOOM:
      // Right-mouse button is being held down
      // Zoom into or away from the scene based upon how far the mouse moved in the x-direction.                 
      //   This implementation does this by scaling the eye-space.                                               
      //   This should be the first operation performed by the GL_PROJECTION matrix.                             
      //   1. Calculate the signed distance                                                                      
      //       a. movement to the left is negative (zoom out).                                                   
      //       b. movement to the right is positive (zoom in).                                                   
      //   2. Calculate a scale factor for the scene s = 1 + a*dx                                                
      //   3. Call glScalef to have the scale be the first transformation.                                       
      //                                                                                                         
      // post("ZOOM %d %f %f\n", x, last_point.x, zoom_factor);                                                  
      pixelDiff  = me.getX() - m_lastPoint[0];
      m_zoomFactor *= 1.0 + pixelDiff * m_ZOOMSCALE;
      //                                                                                                         
      // Set the current point, so the lastPoint will be saved properly below.                                   
      //                                                                                                         
      //glMatrixMode( GL_PROJECTION );                                                                           
      //glScalef( zoom_factor, zoom_factor, zoom_factor );                                                       
      //glMatrixMode( GL_MODELVIEW );                                                                            
      curPoint[0] = me.getX();
      curPoint[1] = me.getY();
      curPoint[2] = 0;
      break;
    default:
      return;  
    }

    //                                                                                                           
    // Save the location of the current point for the next movement.                                             
    //                                                                                                           
    m_lastPoint = curPoint;
  }


  public void onJoyMove (double x, double y) {
    //double pixelDiffX;
    double moveAmountX;
    //double pixelDiffY;
    double moveAmountY;
    //double[] curJoyPoint;

    moveAmountX = x / m_zoomFactor;
    moveAmountY = -y / m_zoomFactor;
    //glMatrixMode( GL_PROJECTION );                                                                             
    //glTranslatef( move_amount_x, move_amount_y, 0.0 );                                                         
    m_center[0] += moveAmountX;  
    m_center[1] += moveAmountY;
    //glMatrixMode( GL_MODELVIEW );                                                                              
  }

  public void onKeyZoom (float amt) {
    m_zoomFactor *= 1.0 + amt * m_ZOOMSCALE;
  }

  public void setWidthHeight(int w, int h) {
    m_viewWidth = w;
    m_viewHeight = h;
  }

}
