/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.view;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.bbn.morph.executer.Update;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.PhysicalModel;


/**
 *  Model for the robot, simply pairs of attributes and their values.
 *  Provides a function to update the values.  
 * 
 * @author fyaman
 *
 */
public class AttributeValueView extends HashMap<RobotAttribute, Measurement> {

  private static final long serialVersionUID = 1L;
  private PhysicalModel m_model;

  public AttributeValueView(PhysicalModel model) {
		super();	
		m_model = model;
		refresh();
	}

  /**
   * Refresh the view by lookup up the attributes in the model.
   * Note that this doesn't clear the HashMap before adding new values so
   * if attributes are removed, this will not remove them. Also, it does
   * not prevent concurrent gets
   */
  public void refresh() {
    if (m_model != null) {
		  Map<String, Measurement> attributeMap = m_model.getAttributeMap();
		  for (String attribute : attributeMap.keySet()) {
        put(new RobotAttribute(attribute), attributeMap.get(attribute));
      }
		}
  }
	
	// Update the robot model by adjusting the value of the attribute
	// Does not check if the delta set contains multiple updates for the same attribute.
	// If so all will be applied in an additive way.
	public void update(Set<Update> delta) {
		for(Update d: delta){
			RobotAttribute attribute = d.getAttribute();
			Measurement old = get(attribute);
			if (old==null){
				System.out.println("Something is wrong -- trying to update an attribute that does not exists: " + attribute);
				continue;
			}
			
			old = old.addMD(d.getVal().getStandardValue());
			put(attribute, old);
				
		}
	}

}
