/*
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.view;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.bbn.morph.model.DesiredPhysicalModel;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.RobotAttributeValues;

/**
 * A view of a desired model
 * @author aadler
 *
 */
public class DesiredAttributeValueView {
  
  private DesiredPhysicalModel m_desiredModel;

  private HashMap<RobotAttribute, Measurement> m_baseAttributes;
  private HashMap<RobotAttribute, Measurement> m_desiredAttributes;
  
  public DesiredAttributeValueView(DesiredPhysicalModel desiredModel) {
    m_desiredModel = desiredModel;
    m_baseAttributes = new HashMap<RobotAttribute, Measurement>();
    m_desiredAttributes = new HashMap<RobotAttribute, Measurement>();
    refresh();
  }
  
  /**
   * Refresh the view by lookup up the attributes in the model.
   * Note that it does
   * not prevent concurrent gets
   */
  public void refresh() {
    Map<String, Measurement> attributeMap = m_desiredModel.getBaseModel().getAttributeMap();
    m_baseAttributes.clear();
    m_desiredAttributes.clear();
    
    for (String attribute : attributeMap.keySet()) {
      m_baseAttributes.put(new RobotAttribute(attribute), attributeMap.get(attribute));
    }
    for (RobotAttributeValues rav : m_desiredModel.getDesiredChanges()) {
      m_desiredAttributes.put(new RobotAttribute(rav.getAttribute()), rav.getMeasurement());
    }
  }
  
  /**
   * @return true iff all of the desired changes have been made
   */
  public boolean areDesiredChangesMade() {
    for (RobotAttribute attribute : m_desiredAttributes.keySet()) {
      if (m_baseAttributes.get(attribute) != m_desiredAttributes.get(attribute))
        return false;
    }
    return true;
  }
  
  /**
   * @param attribute the attribute to look up the value for
   * @return the value to which the specified attribute is mapped, or null if the map contains no mapping for this attribute. 
   */
  public Measurement get(RobotAttribute attribute) {
    return m_desiredAttributes.get(attribute);
  }
  
  public List<RobotAttribute> getAllDesiredAttributes() {
    return new ArrayList<RobotAttribute>(m_desiredAttributes.keySet());
  }

}
