/* Code to draw OpenGL boxes
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.view;

import java.awt.Color;

import javax.media.opengl.GL;
import javax.media.opengl.GL2;
import javax.media.opengl.GLAutoDrawable;

public abstract class OpenGLViewDraw {
	
	public abstract void drawView(GLAutoDrawable drawable);
	
	public void drawBox(double dx, double dy, double dz, double dw, double db, double dh, GL2 gl, Color c) {
		// need floats for OpenGL
		float x = (float) (dx - (dw /2 ));
		float y = (float) (dy - (db / 2));
		float z = (float) (dz - (dh / 2));
		float w = (float) dw;
		float b = (float) db;
		float h = (float) dh;
		
		gl.glPushMatrix();
	    gl.glBegin(GL.GL_TRIANGLES); // Begin drawing triangle sides
	    
	    // Front panel
	    float[] components = c.getComponents(null);
	    
	    gl.glColor4f(components[0], components[1], components[2], components[3]); //( 0, 0.5f, 1 );
	    gl.glVertex3f( x, y, z ); // Origin
	    gl.glVertex3f(x+w, y, z);
	    gl.glVertex3f( x, y+b, z );
	    
	    gl.glVertex3f( x+w, y, z );
	    gl.glVertex3f( x, y+b, z );
	    gl.glVertex3f( x+w, y+b, z );
	    
	    // Back panel
	    gl.glVertex3f( x, y, z+h );
	    gl.glVertex3f( x+w, y, z+h );
	    gl.glVertex3f( x, y+b, z+h );
	    
	    gl.glVertex3f( x+w, y, z+h );
	    gl.glVertex3f( x+w, y+b, z+h );
	    gl.glVertex3f( x, y+b, z+h );
	    
	    // Left panel
	    gl.glVertex3f( x, y+b, z+h );
	    gl.glVertex3f( x, y+b, z );
	    gl.glVertex3f( x, y, z+h );
	    
	    gl.glVertex3f( x, y+b, z );
	    gl.glVertex3f( x, y, z+h );
	    gl.glVertex3f( x, y, z );
	    
	    // Right panel
	    gl.glVertex3f( x+w, y+b, z+h );
	    gl.glVertex3f( x+w, y+b, z );
	    gl.glVertex3f( x+w, y, z+h );
	    
	    gl.glVertex3f( x+w, y+b, z );
	    gl.glVertex3f( x+w, y, z+h );
	    gl.glVertex3f( x+w, y, z );
	    
	    // Bottom panel
	    gl.glVertex3f( x, y, z );
	    gl.glVertex3f( x, y, z+h );
	    gl.glVertex3f( x+w, y, z );
	    
	    gl.glVertex3f( x, y, z+h );
	    gl.glVertex3f( x+w, y, z );
	    gl.glVertex3f( x+w, y, z+h );
	    
	    // Top panel
	    gl.glVertex3f( x, y+b, z );
	    gl.glVertex3f( x, y+b, z+h );
	    gl.glVertex3f( x+w, y+b, z );
	    
	    gl.glVertex3f( x, y+b, z+h );
	    gl.glVertex3f( x+w, y+b, z );
	    gl.glVertex3f( x+w, y+b, z+h );
	    
	    gl.glEnd();
	    //gl.glFlush();
	    gl.glPopMatrix();
	}
	
	public void drawCylinder(double dx, double dy, double dz, double dlength, double dradius, GL2 gl) {
		
	}
}
