/*
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.view;

import java.awt.Color;
import javax.media.opengl.GL2;
import javax.media.opengl.GLAutoDrawable;

import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.ModelBody;
import com.bbn.morph.model.ModelBox;
import com.bbn.morph.model.ModelCylinder;
import com.bbn.morph.model.ModelObject;
import com.bbn.morph.model.ModelTransform;
import com.bbn.morph.model.PhysicalModel;

public class UIPhysicalView extends OpenGLViewDraw {

	private PhysicalModel m_model;
	
	public UIPhysicalView(PhysicalModel model) {
		m_model = model;
	}
	
	@Override
	public void drawView(GLAutoDrawable drawable) {
		GL2 gl = drawable.getGL().getGL2();
		for(ModelBody modelBody : m_model.getModelBodies()) {
			
			gl.glPushMatrix();
			
			ModelObject parent = modelBody.getParentObject();
			
			// If it has a parent, the parent is a transform and should be applied
			while((parent != null) && (parent instanceof ModelTransform )) {
				ModelTransform transform = (ModelTransform) parent;
				gl.glTranslatef((float) transform.getXShift(), (float) transform.getYShift(), (float) transform.getZShift());
				parent = parent.getParentObject();
			}
			
			// Draw the object
			if(modelBody instanceof ModelBox) {
				ModelBox modelBox = (ModelBox) modelBody;
				Measurement w = modelBox.getAttributeValue("lengthx");
				Measurement b = modelBox.getAttributeValue("lengthy");
				Measurement h = modelBox.getAttributeValue("lengthz");
				Measurement x = ((ModelBody) modelBody).getPosition().getxPos();
				Measurement y = ((ModelBody) modelBody).getPosition().getyPos();
				Measurement z = ((ModelBody) modelBody).getPosition().getzPos();
				
				Color c = Color.GREEN;
				if (modelBox.getId().equals("botbody"))
					c = Color.BLUE;
        if (modelBox.getId().equals("motor"))
          c = Color.MAGENTA;
				else if (modelBox.getId().equals("step"))
				  c = Color.ORANGE;
				
				drawBox(x.getStandardValue(), y.getStandardValue(), z.getStandardValue(), w.getStandardValue(), b.getStandardValue(), h.getStandardValue(), gl, c);
			}
			else if(modelBody instanceof ModelCylinder) {
				Measurement len = ((ModelCylinder) modelBody).getLength();
				Measurement rad = ((ModelCylinder) modelBody).getRadius();
				Measurement x = ((ModelBody) modelBody).getPosition().getxPos();
				Measurement y = ((ModelBody) modelBody).getPosition().getyPos();
				Measurement z = ((ModelBody) modelBody).getPosition().getzPos();
				
				drawCylinder(x.getStandardValue(), y.getStandardValue(), z.getStandardValue(), len.getStandardValue(), rad.getStandardValue(), gl);
			}
			// TODO spheres
			
			gl.glPopMatrix();
		}

	}

}
