/*
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */

package com.bbn.morph.view.test;

import java.util.HashMap;
import java.util.Set;

import junit.framework.Assert;

import org.junit.Test;

import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.Pair;
import com.bbn.morph.executer.StressInputType;
import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.executer.Update;
import com.bbn.morph.view.FB;
import com.bbn.morph.view.ThreePieceLinearStressFunction;
import com.bbn.morph.view.RobotAttribute;
import com.bbn.morph.view.StepDecrementUpdate;
import com.bbn.morph.view.StepIncrementUpdate;
import com.bbn.morph.view.StressFunction;
import com.bbn.morph.view.UpdateFunction;

public class FBTest {

	public StressInputs genInput(String FBname, StressInputType inputLabel, double val){
		StressInputs inputs = new StressInputs(new Feature(FBname));
		inputs.put(inputLabel, val);
		return inputs;
	}
	
	@Test
	public void testFB(){
		Pair first =new Pair(3,1);
		Pair second =new Pair(5,3);
		StressInputType inputLabel= StressInputType.TIME;
		StressFunction stressF= new ThreePieceLinearStressFunction(inputLabel, first, second);
		UpdateFunction updateF1= new StepDecrementUpdate(0.5);
		UpdateFunction updateF2= new StepIncrementUpdate(0.6);
		HashMap<RobotAttribute, UpdateFunction> updateFunctions = new HashMap<RobotAttribute, UpdateFunction>();
		updateFunctions.put(new RobotAttribute("FlipperSize"), updateF1);
		updateFunctions.put(new RobotAttribute("BodyWeight"), updateF2);
		FB testFB = new FB(new Feature("testFB"), updateFunctions, stressF);
		StressInputs inputs1 = genInput("testFB", inputLabel, 4);
		Double stress = stressF.computeStress(inputs1);
		Set<Update> updates = testFB.updates(stress);
		System.out.print(updates);
		Assert.assertEquals(updates.size(), 2);
		for(Update u: updates){
			if (u.getAttribute().getName().equals("FlipperSize"))
				Assert.assertEquals(u.getVal().getStandardValue(), 0.25);
			if (u.getAttribute().getName().equals("BodyWeight"))
				Assert.assertEquals(u.getVal().getStandardValue(), -0.3);
		}
	}

	public static  void main(String[] args) {
		
	}

	
}
