/* 
Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
in the AUTHORS file in the MADV distribution's top directory.

This file is part of MADV, and is distributed under the terms of the
GNU General Public License, with a linking exception, as described in
the file LICENSE in the MADV distribution's top directory. */
package com.bbn.morph.view.test;
import junit.framework.Assert;

import org.junit.Test;

import com.bbn.morph.controller.test.FirstRobotModel;
import com.bbn.morph.executer.Feature;
import com.bbn.morph.executer.StressInputType;
import com.bbn.morph.executer.StressInputs;
import com.bbn.morph.model.CompositeModel;
import com.bbn.morph.model.InvalidModelException;
import com.bbn.morph.model.Measurement;
import com.bbn.morph.model.PhysicalModel;
import com.bbn.morph.model.Units;
import com.bbn.morph.model.test.FBTestGenerator;
import com.bbn.morph.view.RelativeIncrementDecrementUpdate;
import com.bbn.morph.view.RobotAttribute;
import com.bbn.morph.view.StepDecrementUpdate;
import com.bbn.morph.view.StepIncrementUpdate;
import com.bbn.morph.view.UpdateFunction;

public class UpdateFunctionTest {

	public StressInputs genInput(String FBname, StressInputType inputLabel, double val){
		StressInputs inputs = new StressInputs(new Feature(FBname));
		inputs.put(inputLabel, val);
		return inputs;
	}
	
	@Test
	public void testStepIncrFunction(){
		UpdateFunction updateF = new StepIncrementUpdate(6);
		System.out.println("Testing:"+ updateF.description());
		Assert.assertEquals(updateF.delta(0.5),3.0);
		Assert.assertEquals(updateF.delta(-2),-12.0);
	};
	
	@Test
	public void testStepDecrFunction(){
		UpdateFunction updateF = new StepDecrementUpdate(6);
		System.out.println("Testing:"+ updateF.description());
		Assert.assertEquals(updateF.delta(0.5),-3.0);
		Assert.assertEquals(updateF.delta(-2),12.0);
	};


	@Test
	public void testRelativeIncUpdateFunction() throws InvalidModelException{
	  CompositeModel compositeModel = FirstRobotModel.getCompositeModel();
	  PhysicalModel model = compositeModel.getRobotModel();
		RobotAttribute bodyAttr = new RobotAttribute(FBTestGenerator.bodyLengthAttribute);
		
		// needs to reach 4 from 3 with 10% increments
		UpdateFunction updateF = new RelativeIncrementDecrementUpdate(0.1,model,bodyAttr,new Measurement(Units.getMeters(),4.0));
		System.out.println("Testing:"+ updateF.description());
		Assert.assertEquals(updateF.delta(0.5),0.1*3.0);
		Assert.assertEquals(updateF.delta(2),0.1*3.0);
		
		// needs to reach 3.1 from 3 with 10% increments
		updateF = new RelativeIncrementDecrementUpdate(0.1,model,bodyAttr,new Measurement(Units.getMeters(),3.1));
		System.out.println("Testing:"+ updateF.description());
		Assert.assertEquals(updateF.delta(0.5),3.1-3);
		
		
		// needs to decrease to 2.5 from 3 with 10% increments
		updateF = new RelativeIncrementDecrementUpdate(0.1,model,bodyAttr,new Measurement(Units.getMeters(),2.5));
		System.out.println("Testing:"+ updateF.description());
		Assert.assertEquals(updateF.delta(-1),-0.1*3.0);
		
		
	};

	
	
	public static  void main(String[] args) {
		
	}

	
}
