;; Copyright (C) 2013-2014, Raytheon BBN Technologies and contributors listed
;; in the AUTHORS file in the MADV distribution's top directory.

;; This file is part of MADV, and is distributed under the terms of the
;; GNU General Public License, with a linking exception, as described in
;; the file LICENSE in the MADV distribution's top directory. 

(progn
  (setf show-cells nil)
  (setf show-values-in-parameters t))
;;  (setf show-priority-values t))

;; for a test, change the value of tetralander instrument package
(compile-and-load "tetralander")
(marked-values 
 `((INSTRUMENT_PACKAGE_VOLUME 0.01)	; 20x
   (INSTRUMENT_PACKAGE_MASS 5)))	; 10x
;; new idea... adjust downstream only... mark each p/v with a hopcount
(setf (enabled (find-relation "instrument mass/volume")) nil)

(setf chart (diagnose-and-repair))
(let ((header "% Tetrahedral lander, with the following changes:~%~
              % instrument package volume increased 20x, mass increased 10x~%~
              % instrument mass/volume relation disabled~%")
      (filename "tetralander_test1"))
  (output-diagnose-and-repair-graph chart "tet_1_" filename "graphs/" header))

(human-readable-conflict-report (diagnose t))
(run-batches-until-stuck)

(output-graphviz "tetralander.dot" (/ 11 8.5))


;; another test case, changing the solar cells too
(compile-and-load "tetralander")
(marked-values 
 `((INSTRUMENT_PACKAGE_POWER 10)	; 20x
   (INSTRUMENT_PACKAGE_VOLUME 0.0005)	; base case
   (INSTRUMENT_PACKAGE_MASS 0.5)))	; base case
(setf (enabled (find-relation "instrument mass/volume")) nil)

(diagnose-and-repair)
(human-readable-conflict-report (diagnose t))
(dotimes (i 10)
  (run-batches-until-stuck)
  (shift-resolution-focus)
  (explain-current-priorities)
  )

(output-graphviz "tetralander.dot" (/ 11 8.5))

;; orginal test case, changing instrument power and mass by 6x
(progn
  (setf step-size 0.005)
  (compile-and-load "tetralander")
  (marked-values 
   `((INSTRUMENT_PACKAGE_POWER 3)
     (INSTRUMENT_PACKAGE_MASS 3)))
  (human-readable-conflict-report (diagnose t))
  (run-with-topochange-until-stuck))
;; it will get stuck with the bound on the motor component, split the motor
(human-readable-conflict-report (diagnose t))
(apply-best-candidate-change)
(run-batches-until-stuck)
(run-with-topochange-until-stuck)

(output-graphviz "tetralander.dot" (/ 11 8.5))

;; and turn it back down to see if merge works...
(progn
  (marked-values 
   `((BOT_BODY_MASS 0.5)
     (INSTRUMENT_PACKAGE_POWER 0.1)
     (INSTRUMENT_PACKAGE_MASS 0.1)))
  (human-readable-conflict-report (diagnose t))
  (run-with-topochange-until-stuck))

(output-graphviz "tetralander.dot" (/ 11 8.5))

;; solar cell case, changing topology
(progn
  (setf step-size 0.005)
  (compile-and-load "tetralander")
  (marked-values 
   `((INSTRUMENT_PACKAGE_POWER 10)	; 20x
     (INSTRUMENT_PACKAGE_VOLUME 0.0005)	; base case
     (INSTRUMENT_PACKAGE_MASS 0.5)))	; base case
  (setf (enabled (find-relation "instrument mass/volume")) nil)
  (human-readable-conflict-report (diagnose t))
  (run-with-topochange-until-stuck))

(run-with-topochange-until-stuck)
(output-graphviz "tetralander.dot" (/ 11 8.5))



;; minidroid topology change test:
;; solar cell case, changing topology
(progn
  (setf step-size 0.005 autoadjust t)
  (compile-and-load "minidroid")
  (mapcar (ppf mark ?x 'priority 0) (all-elements)) ; speedup
  (marked-values
   '((step_height 0.25)))
  (diagnose t)
  (run-with-topochange-until-stuck 10))

(run-with-topochange-until-stuck 10)
(setf use-neato nil)
(output-graphviz "minidroit.dot" (/ 11 8.5))


; Future ideas:
; - determine difference between "within component" and "cross-component". 
;  - Option: adjust only "within component" for initial diagnosis?
;  - automatic identification of marked values from initial diagnosis
;  - create a chained explanation of a problem
