;; Copyright (C) 2013-2014, Raytheon BBN Technologies and contributors listed
;; in the AUTHORS file in the MADV distribution's top directory.

;; This file is part of MADV, and is distributed under the terms of the
;; GNU General Public License, with a linking exception, as described in
;; the file LICENSE in the MADV distribution's top directory. 

;; Tests:
;; Rectangle:
(clear-topology)
(let ((x (make-dimension 'x 'meters)) (y (make-dimension 'y 'meters))
      (point (make-cell 0)))
  (extrude (extrude point x) y))
(output-graphviz)

;; Box:
(clear-topology)
(extrude-box)
(make-value (find-parameter 'x) 2)
(make-value (find-parameter 'y) 0.3)
(make-value (find-parameter 'xyz) 1.5)
(make-value (find-parameter 'z) 2)	; mismatch
(resolve-all)
(output-graphviz)

;; Rectangle with one extrusion:
(clear-topology)
(let ((r (extrude-rectangle)))
  (extrude (first (boundaries r)) (make-dimension '|Distal| 'meters)))
(output-graphviz)

;; Box with one extrusion:
(clear-topology)
(let ((b (extrude-box)))
  (extrude (first (boundaries b)) (make-dimension '|Distal| 'meters)))
(output-graphviz)

;; Box with two extrusions:
(clear-topology)
(let ((b (extrude-box)) (dp (make-dimension '|Distal| 'meters)))
  (extrude (append (find-cells '((dimension ?c 2)) '((x +)))
		   (find-cells '((dimension ?c 2)) '((x -))))
	   dp))
(output-graphviz)
(progn					; assign values
  (make-value (find-parameter 'x) 4.0)
  (make-value (find-parameter 'y) 3.0)
  (make-value (find-parameter 'z) 2.0)
  (make-value (find-parameter '|Distal|) 1.5))
(coordinatize-vertices '(x y z))
(output-mesh)
;; An important oddidy to note: when you extrude two attached cells, it
;; doesn't care if they were intended to be orthogonal.  You end up
;; with an attachment in the extruded cell as well.
;; This is *good*, and looks forward to the non-orthogonal future.

;; Cube
(clear-topology)
(let ((point (make-cell 0)) (x (make-dimension 'x 'meters)))
  (extrude (extrude (extrude point x) x) x))
(output-graphviz)

;; Sub-lines
(clear-topology)
(let ((x (make-dimension 'x 'meters)) (point (make-cell 0)))
  (subspace (extrude point x) '((x ? sub ?))))
(output-graphviz)
;; subspace relationship between 1-cells only

(clear-topology)
(let ((x (make-dimension 'x 'meters)) (point (make-cell 0)))
  (subspace (extrude point x) '((x lgap sub rgap))))
(output-graphviz)

(clear-topology)
(let ((x (make-dimension 'x 'meters)) (point (make-cell 0)))
  (subspace (extrude point x) '((x gap sub gap))))
(output-graphviz)

(clear-topology)
(let* ((x (make-dimension 'x 'meters)) (point (make-cell 0))
       (l (extrude point x)))
  (subspace l '((x nil sub gap)))) ; share point
(output-graphviz)
;; subspace relationship between 1-cells only

;; Sub-rectangles
(clear-topology)
(let ((r (extrude-rectangle)))
  (subspace r '((x ? subx ?) (y ? suby ?)))) ; share nothing
(output-graphviz)
;; subspace relationship between 2-cells only

(clear-topology)
(let ((r (extrude-rectangle)))
  (subspace r '((x gap sub gap) (y gap sub gap)))) ; symmetric gap
(progn					; assign values
  (make-value (find-parameter 'x) 1.0)
  (make-value (find-parameter 'y) 1.5)
  (make-value (find-parameter 'sub) 0.3)
  (make-value (find-parameter 'gap) 0.4))
(resolve-all)
(output-graphviz)
;; subspace relationship between 2-cells only

(clear-topology)
(let ((r (extrude-rectangle)))
  (subspace r '((x nil subx ?) (y nil suby ?))))
(progn					; assign values
  (make-value (find-parameter 'x) 1.0)
  (make-value (find-parameter 'y) 1.5)
  (make-value (find-parameter 'subx) 0.3)
  (make-value (find-parameter 'suby) 0.4))
(output-graphviz)
;; subspace relationship between 2-cells, 1-cells on shared point

(clear-topology)
(let ((r (extrude-rectangle)))
  (subspace r (sub-parameters '((x sub) (y sub))) ; symmetry in sub
	    ;; share -x,-y point
	    (first (find-elements '((type ?c cell) (dimension ?c 0)
				    (-> x minimum ?c) (-> y minimum ?c))))))
(output-graphviz)
;; subspace relation/ship between 2-cells, 1-cells on shared point

(clear-topology)
(let ((r (extrude-rectangle)))
  (subspace r (sub-parameters '(x (y sub)))
	    (find-if (lambda (c) (let ((l (car (links-of-type c 'minimum t))))
				   (and l (eq (tail l) (find-parameter 'y)))))
		     (boundaries r))))	; share -Y line
;;	    (first			; share -Y line
;;	     (find-elements '((type ?p cell) (dimension ?p 2) (y minimum ?p))))))
(output-graphviz)
;; subspace relationship between 2-cells, 1-cells adjacent to line

(clear-topology)
(let ((r (extrude-rectangle)))
  (subspace r (sub-parameters '(x (y sub)))))	; share parallel lines
(output-graphviz)
;; subspace relationship between 2-cells, 1-cells on y-axis

;; Extrude two edges simultaneously
(clear-topology)
(let ((r (extrude-rectangle)))
  (extrude (find-elements '((type ?c cell) (dimension ?c 1) (-> x extremum ?c)))
	   (make-dimension 'foo 'meters)))
(output-graphviz)

;; Extrude four edges simultaneously
(clear-topology)
(let ((r (extrude-rectangle)))
  (extrude (find-elements '((type ?c cell) (dimension ?c 1)))
	   (make-dimension 'foo 'meters)))
(output-graphviz)

;; Find four subspaces simultaneously
(clear-topology)
(let ((r (extrude-rectangle)))
  (subspace r (sub-parameters '((x sub) (y sub))) ; symmetry in sub
	    ;; share corners
	    (find-elements '((type ?c cell) (dimension ?c 0)))))
(output-graphviz)
;; subspace relation/ship between 2-cells, 1-cells on shared point



;; Table (not yet complete)
(clear-topology)
(let ((top (extrude-box)))
  (let* ((underside
	  (find-if (lambda (c) (let ((l (car (links-of-type c 'minimum t))))
				 (and l (eq (tail l) (find-parameter 'z)))))
		   (boundaries top)))
	 (corners (remove-if-not (lambda (c) (= (dimension c) 0))
				 (boundaries* underside)))
	 (l (make-dimension 'leg 'meters)))
    (dolist (c corners)
      (extrude (subspace underside (sub-parameters '((x |LegW|) (y |LegW|))) c) 
	       l))
    ;; set up FBs:
    ;; Pine has 4,800 psi compressive strength, specific gravity 0.35
    ;; Strength is thus: 3.3e7 Pascals = N/m^2
    ;; Weight = volume * 0.35 
    ;; Actually, I need buckling strength... estimate 10x lower?
    (let ((3cells (remove-if-not (lambda (e) (and (typep e 'cell) (= (dimension e) 3))) (all-elements)))
	  (legbottoms (remove-if-not 
		       (lambda (e) (and (typep e 'cell) (= (dimension e) 2)
					(let ((l (car (links-of-type e 'maximum t))))
					  (and l (eq (tail l) (find-parameter 'leg))))))
		       (all-elements))))
      (make-relieve-pressure 3cells legbottoms 0.35 3.3e6))
    ))
(progn					; assign values
  (make-value (find-parameter 'x) 1.5)
  (make-value (find-parameter 'y) 0.5)
  (make-value (find-parameter 'z) 0.10)	; 10cm thick
  (make-value (find-parameter '|LegW|) 0.002) ; 2 mm legs
  (make-value (find-parameter 'leg) 1))	; 1 meter leg
(resolve-all)
(setf show-cells t)
(output-graphviz)
(setf show-cells nil)
(output-graphviz "blueprint.dot" 1.5)

(defclass relieve-pressure (functional-blueprint)
  ((load :initarg :load)		; set of 3-cell volumes
   (contact :initarg :contact)		; set of 2-cell areas
   (density :initarg :density)
   (strength :initarg :strength)))
(defun make-relieve-pressure (load-cells contact-cells density strength)
  (assert (and (>= (length load-cells) 1) (>= (length contact-cells) 1)))
  (let ((areas (mapcar (pcf find-measure '(meters meters)) contact-cells))
	(volumes (mapcar (pcf find-measure '(meters meters meters)) load-cells)))
    (assert (not (position nil (append areas volumes))))
    (let ((rp (make-instance 'relieve-pressure :load volumes :contact areas
			     :density density :strength strength)))
      (dolist (p (append areas volumes))
	(make-constraint rp p)))))

;; TODO: hook this up to a posited load requirement
(defparameter g 9.80665)
(defparameter safety-margin 2.0)
(defmethod resolve ((rp relieve-pressure))
  (with-slots (load contact density strength) rp
    (let ((volumes (mapcar #'get-values load))
	  (areas (mapcar #'get-values contact)))
      (setf (stress rp) 0)
      (unless (position-if (lambda (l) (not (= (length l) 1))) 
			   (append volumes areas)) ; if all are set
	(setf volumes (mapcar (cmp value first) volumes))
	(setf areas (mapcar (cmp value first) areas))
	(let* ((mass (* density 1000 (apply #'+ volumes))) ; water = 1000 kg/m^2
	       (weight (* mass g))
	       (pressure (/ weight (apply #'+ areas))))
	  ;;(format t "Pressure is: ~A (strength = ~A)" pressure strength)
	  (let* ((margin (/ strength pressure)))
	    (setf (stress rp) (max 0 (- safety-margin margin)))
	    (when (> (stress rp) 0)
	      (mapcar (ppf request-change ?a '+) contact)
	      (mapcar (ppf request-change ?a '-) load))))))))

(find-elements '((type ?x cell) (dimension ?x 2) (-> z minimum ?x)))


;;;;;;;;;;;;;;;;
;; Compressed table:

(clear-topology)
(let ((top (extrude-box)))
  (let* ((underside 
	  (first (find-elements '((type ?x cell) (dimension ?x 2) (-> z minimum ?x)))))
	 (corners 
	  (find-elements `((boundaries* ,underside ?x) (dimension ?x 0))))
	 (l (make-dimension 'leg 'meters)))
    (dolist (c corners)
      (extrude (subspace underside (sub-parameters '((x |LegW|) (y |LegW|))) c) 
	       l))
    ;; set up FBs:
    ;; Pine has 4,800 psi compressive strength, specific gravity 0.35
    ;; Strength is thus: 3.3e7 Pascals = N/m^2
    ;; Weight = volume * 0.35 
    ;; Actually, I need buckling strength... estimate 10x lower?
    (let ((3cells (find-elements '((type ?x cell) (dimension ?x 3))))
	  (legbottoms (find-elements 
		       '((type ?x cell) (dimension ?x 2) (-> leg maximum ?x)))))
      (make-relieve-pressure 3cells legbottoms 0.35 3.3e6))
    ))
(progn					; assign values
  (make-value (find-parameter 'x) 1.5)
  (make-value (find-parameter 'y) 0.5)
  (make-value (find-parameter 'z) 0.10)	; 10cm thick
  (make-value (find-parameter '|LegW|) 0.02) ; 2 cm legs
  (make-value (find-parameter 'leg) 1))	; 1 meter leg
(resolve-all)
(setf show-cells t)
(output-graphviz)
(setf show-cells nil)
(output-graphviz "blueprint.dot" 1.5)
(make-python-file)

;;;;;;;;;;;;;;;;
;; Compressed chair:

(clear-topology)
(let ((top (extrude-box)))
  (let* ((underside 
	  (first (find-elements '((type ?x cell) (dimension ?x 2) (-> z minimum ?x)))))
	 (corners 
	  (find-elements `((boundaries* ,underside ?x) (dimension ?x 0))))
	 (l (make-dimension 'leg 'meters)))
    (dolist (c corners)
      (extrude (subspace underside (sub-parameters '((x |LegW|) (y |LegW|))) c) 
	       l)))
  (let* ((topside
	  (first (find-elements '((type ?x cell) (dimension ?x 2) (-> z maximum ?x)))))
	 (upback
	  (first (find-elements `((boundaries* ,topside ?x) (dimension ?x 1) (-> x maximum ?x)))))
	 (b (make-dimension 'back 'meters)))
    (extrude (subspace topside (sub-parameters '((x |BackW|) (y y))) upback)
	     b)))
(progn					; assign values
  (make-value (find-parameter 'x) 0.55)
  (make-value (find-parameter 'y) 0.6)
  (make-value (find-parameter 'z) 0.3)	; 10cm thick
  (make-value (find-parameter '|LegW|) 0.02) ; 2 cm legs
  (make-value (find-parameter 'leg) 0.5) ; 1 meter leg
  (make-value (find-parameter '|BackW|) 0.03)
  (make-value (find-parameter 'back) 0.7))
(resolve-all)
(setf show-cells t)
(output-graphviz "chair.dot" 1.5)
(setf show-cells nil)
(output-graphviz "chair-reduced.dot" 1.5)
(make-python-file)

;;;;;;;;; 
;; Initial minidroid

(clear-topology)
(progn
  (setf egg 
    (extrude-box 'bot_body_length_x 'bot_body_length_y 'bot_body_length_z))
  (setf sides (find-elements '((type ?x cell) (dimension ?x 2)
			       (-> 'bot_body_length_x extremum ?x))))
  (setf wheel-corners (find-elements '((type ?x cell) (dimension ?x 1)
			       (boundary* ,sides ?x))))
  ;; create wheels
  (setf wheels 
    (extrude 
     (subspace sides (sub-parameters '((bot_body_length_z bot_body_length_z)
				       (bot_body_length_y bot_body_length_z)))
	       wheel-corners)
     (make-dimension 'wheel_depth 'meters)))
  ;; find flipper axle within rear wheel
  (subspace rear-wheels '((bot_body_length_z flipper_length_z)
			  (bot_body_length_y flipper_length_z)))
  ;; extrude flipper axle
  (extrude (find-elements '((dimension ?x 2) (-> wheel_depth maximum ?x))
			  (boundaries* flipper-axle))
	   (make-dimension 'flipper_length_x))
  ;; extrude flipper body
  (extrude flipper-rear (make-dimension 'flipper_length_z))
  )
;; 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;; GDS 2013 Paper
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defclass solar-generation (relation)
  ((area :initarg :area)
   (peak-power :initarg :power)
   (efficiency :initarg :eff))) ; peak Watts/m^2

(defun make-solar-panel (area power eff)
  (let ((sp (make-instance 'solar-generation :area area :power power :eff eff)))
    (make-constraint sp area)
    (make-constraint sp power)
    (make-constraint sp eff)
    sp))

;;;;;
;; Solar panel
;;;;;
(clear-topology)
(extrude-rectangle)
(make-solar-panel (find-parameter 'xy) (make-parameter 'peak-power 'Watts)
		  (make-parameter 'power-density 'Watts/m/m))
(make-value (find-parameter 'y) 3)
(make-value (find-parameter 'power-density) 150)
(make-value (find-parameter 'peak-power) 2000)
(get-topology-stats)
(output-graphviz "blueprint.dot" 1.5)

;;;;;
;; Solar panel on roof
;;;;;
(clear-topology)
(setf seed (make-cell 0))
(output-graphviz "blueprint.dot")
(setf line (extrude seed (make-dimension 'roof_x 'meters)))
(output-graphviz "blueprint.dot" 1.5)
(setf roof (extrude line (make-dimension 'roof_y 'meters)))
(output-graphviz "blueprint.dot" 1.5)
(subspace roof '((roof_x ? x ?) (roof_y ? y ?)))
(output-graphviz "blueprint.dot" 1.5)
(make-solar-panel (find-parameter 'xy) (make-parameter 'peak-power 'Watts)
		  (make-parameter 'power-density 'Watts/m/m))
(output-graphviz "blueprint.dot" 1.5)
(make-value (find-parameter 'roof_x) 10)
(make-value (find-parameter 'roof_y) 4)
(make-value (find-parameter 'y) 3)
(make-value (find-parameter 'power-density) 150)
(make-value (find-parameter 'peak-power) 2000)
(get-topology-stats)
(output-graphviz "blueprint.dot" 1.5)

;; For test output without resolution
(make-value (find-parameter 'x) 5)

;;;;;
;; 4-legged Table
;;;;;
(clear-topology)
(let ((top (extrude-box)))
  (setf underside (car (find-cells '((dimension ?c 2)) '((z -)))))
  (setf l (make-dimension '|Leg| 'meters))
  (extrude (subspace underside '((x nil |LegW| ?) (y nil |LegW| ?))) l)
  (extrude (subspace underside '((x ? |LegW| nil) (y nil |LegW| ?))) l)
  (extrude (subspace underside '((x nil |LegW| ?) (y ? |LegW| nil))) l)
  (extrude (subspace underside '((x ? |LegW| nil) (y ? |LegW| nil))) l)
  )
(progn					; assign values
  (make-value (find-parameter 'x) 1.5)
  (make-value (find-parameter 'y) 0.5)
  (make-value (find-parameter 'z) 0.05)	; 5cm thick
  (make-value (find-parameter '|LegW|) 0.05) ; 5 cm legs
  (make-value (find-parameter '|Leg|) 1)) ; 1 meter leg
(get-topology-stats)
(output-graphviz "blueprint.dot" 1.5)

;;;;;;
;; 4-legged chair - starts with table
;;;;;;
(clear-topology)
(let ((top (extrude-box)))
  (setf underside (car (find-cells '((dimension ?c 2)) '((z -)))))
  (setf l (make-dimension '|Leg| 'meters))
  (extrude (subspace underside '((x nil |LegW| ?) (y nil |LegW| ?))) l)
  (extrude (subspace underside '((x ? |LegW| nil) (y nil |LegW| ?))) l)
  (extrude (subspace underside '((x nil |LegW| ?) (y ? |LegW| nil))) l)
  (extrude (subspace underside '((x ? |LegW| nil) (y ? |LegW| nil))) l)
  )
(progn
  (setf topside (car (find-cells '((dimension ?c 2)) '((z +)))))
  (setf b (make-dimension '|Back| 'meters))
  (extrude (subspace topside '((x nil |BackW| ?) (y nil y nil))) b)
  )
(progn
  (make-value (find-parameter 'x) 0.5)
  (make-value (find-parameter 'y) 0.6)
  (make-value (find-parameter 'z) 0.03)	; 3cm thick
  (make-value (find-parameter '|LegW|) 0.03) ; 3 cm legs
  (make-value (find-parameter '|Leg|) 0.5)	; 1/2 meter leg
  (make-value (find-parameter '|Back|) 0.6)
  (make-value (find-parameter '|BackW|) 0.03))
(get-topology-stats)
(setf show-cells nil)
(output-graphviz "blueprint.dot" 1.5)
;;;;;;
;;; Debug point: 68 1-cells, 40 0-cells: should be 71 and 42
;;;;;

;;;;;;
;; Cross
;;;;;;
(clear-topology)
(let ((base (extrude-box)))
  (let ((hwalls (append (find-cells '((dimension ?c 2)) '((x +)))
			(find-cells '((dimension ?c 2)) '((x -)))))
	(v+wall (find-cells '((dimension ?c 2)) '((y +))))
	(v-wall (find-cells '((dimension ?c 2)) '((y -)))))
    (extrude hwalls (make-dimension 'h-bar 'meters))
    (extrude v+wall (find-parameter 'h-bar)) ; up same as horizontal
    (extrude v-wall (make-dimension 'down-bar 'meters))))
(progn					; assign values
  (make-value (find-parameter 'x) 0.1)
  (make-value (find-parameter 'y) 0.1)
  (make-value (find-parameter 'z) 0.1)
  (make-value (find-parameter 'h-bar) 0.5)
  ;;(make-value (find-parameter 'up-bar) 0.5)
  (make-value (find-parameter 'down-bar) 2)) ; 1 meter leg
(get-topology-stats)

;;;;;;
;; 10-DOF robot arm
;;;;;;
(clear-topology)
(let ((base (extrude-box)))
  (setf topside (car (find-cells '((dimension ?c 2)) '((z +)))))
  (setf a (make-dimension '|Arm| 'meters))
  (setf arm
    (extrude (subspace topside '((x gap |ArmW1| gap) (y gap |ArmW2| gap))) a))
  (setf upperarm
    (subspace arm '((|Arm| nil upper-arm lower-arm) (|ArmW1| nil |ArmW1| nil)
		    (|ArmW2| nil |ArmW2| nil))))
  (setf shouldermotor 
    (subspace upperarm '((upper-arm nil s-motor-l ?) (|ArmW1| nil |ArmW1| nil)
		    (|ArmW2| nil |ArmW2| nil))))
  (setf lowerarm
    (subspace arm '((|Arm| upper-arm lower-arm nil) (|ArmW1| ? |LowArmW1| ?)
		    (|ArmW2| ? |LowArmW2| ?)))))
(get-topology-stats)
(progn
  (setf elbowmotorA
    (subspace upperarm '((upper-arm ? e-motor-l nil) (|ArmW1| nil |ArmW1| nil)
		    (|ArmW2| nil |ArmW2| nil))))
  (setf elbowmotorB
    (subspace lowerarm '((lower-arm nil e-motor-l ?)
			 (|LowArmW1| nil |LowArmW1| nil)
			 (|LowArmW2| nil |LowArmW2| nil))))
  (setf forearm
    (subspace lowerarm '((lower-arm nil forearm hand) (|LowArmW1| nil |LowArmW1| nil)
			 (|LowArmW2| nil |LowArmW2| nil))))
  (setf hand
    (subspace lowerarm '((lower-arm forearm hand nil) (|LowArmW1| nil |LowArmW1| nil)
			 (|LowArmW2| nil |LowArmW2| nil)))))
(get-topology-stats)
(progn
  (setf wristmotorA
    (subspace forearm '((forearm ? w-motor-l nil) (|LowArmW1| nil |LowArmW1| nil)
		    (|LowArmW2| nil |LowArmW2| nil))))
  (setf wristmotorB
    (subspace hand '((hand nil w-motor-l ?) (|LowArmW1| nil |LowArmW1| nil)
		    (|LowArmW2| nil |LowArmW2| nil))))
  (setf grippermotor
    (subspace hand '((hand ? g-motor-l nil) (|LowArmW1| nil |LowArmW1| nil)
		     (|LowArmW2| nil |LowArmW2| nil)))))
(get-topology-stats)
(progn
  (setf handfront (car (find-cells '((dimension ?c 2)) '((hand +)))))
  (setf gripper-l (make-dimension 'gripper 'meters))
  (extrude (subspace handfront '((|LowArmW1| nil |LowArmW1| ?)
				 (|LowArmW2| nil |LowArmW2| nil)))
	   gripper-l)
  (extrude (subspace handfront '((|LowArmW1| ? |LowArmW1| nil)
				 (|LowArmW2| nil |LowArmW2| nil)))
	   gripper-l)
  )
(get-topology-stats)
