function plot_joint_test_graph(testid,n_tests)

% Load all data
clear stress shifts designs converged; 
for i=1:n_tests
    eval(sprintf('tetralander_test%i_%i_data;',testid,i-1));
    eval(sprintf('stress(i,1:numel(tet_%i_%i_stress)) = tet_%i_%i_stress;',testid,i-1,testid,i-1));
    eval(sprintf('shifts(i,1:numel(tet_%i_%i_shifts)) = tet_%i_%i_shifts;',testid,i-1,testid,i-1));
    eval(sprintf('converged(i) = tet_%i_%i_stress(end) == 0;',testid,i-1));
    eval(sprintf('tetralander_test%i_%i_design;',testid,i-1));
    designs(i,:) = params;
end
eval(sprintf('tetralander_test%i_initial_design;',testid));
initial_design = params;
maxtime = size(stress,2);

n_converged = sum(converged);
fprintf('%i of %i converged\n',n_converged,n_tests);

convtimes = zeros(n_converged,1);
convdesigns = zeros(n_converged,size(designs,2));
n = 0;
for i=1:n_tests, 
    if converged(i), 
        n=n+1; 
        convdesigns(n,:) = designs(i,:); 
        convtimes(n,:) = find(stress(i,:)==0,1);
    end;
end

% stress vs. time plots
stack = max(max(stress));
h = figure;
for i=1:n_tests
    endtime = find(stress(i,:)==0,1);
    if numel(endtime)==0, endtime = maxtime; end;
    if converged(i), color = 'b'; else color = 'r'; end;
    plot(1:endtime,stress(i,1:endtime),color); hold on;
end
xlabel('Iteration'); ylabel('Absolute stress');
xlim([0 maxtime]); ylim([0 stack]);
title('Stress vs. Time');
outputfig(h,sprintf('tet-%i-stress-chartall',testid));

h = figure;
for i=1:n_tests
    endtime = find(stress(i,:)==0,1);
    if numel(endtime)==0, endtime = maxtime; end;
    if converged(i), color = 'b'; else color = 'r'; end;
    plot(1:endtime,stress(i,1:endtime)+((i-1)*stack),color); hold on;
end
xlabel('Iteration'); ylabel('Absolute stress (by experiment)');
xlim([0 maxtime]); ylim([0 stack*i]);
set(gca,'YTick',[]);
title('Stress charts for individual experiments');
outputfig(h,sprintf('tet-%i-stress-charts',testid));


avg = mean(stress,1);
window = 10000;
smoothed_avg = conv(avg,ones(window,1)/window,'same');

h = figure;
plot(1:maxtime,avg,'r:'); hold on;
plot(1:maxtime,smoothed_avg,'b--');
xlabel('Iteration'); ylabel('Mean total absolute stress');
legend('Mean','Smoothed Mean');
title('Mean Stress');
outputfig(h,sprintf('tet-%i-mean-stress',testid));

% design parameters
h = figure;
semilogy(initial_design,'go'); hold on;
errorbar(mean(designs,1),2*std(designs,1),'*');
set(gca,'YScale','log');
xlabel('Parameter #'); ylabel('Value');
legend('Location','Best','Original','Adjusted');
title('Distribution of All Parameter Values');
outputfig(h,sprintf('tet-%i-all-params',testid));

% converged design parameters
h = figure;
semilogy(initial_design,'go'); hold on;
errorbar(mean(convdesigns,1),2*std(convdesigns,1),'*');
set(gca,'YScale','log');
xlabel('Parameter #'); ylabel('Value');
legend('Location','Best','Original','Adjusted');
title('Distribution of Converged Parameter Values');
outputfig(h,sprintf('tet-%i-converged-params',testid));
% add non-converged as 'outliers'
for i=1:n_tests
    if ~converged(i),
        endtime = find(stress(i,:)==0,1);
        if numel(endtime)==0, endtime = maxtime; end;
        plot(designs(i,:),'r+');
    end
end
legend('Location','Best','Original','Adjusted','Non-Converged');
outputfig(h,sprintf('tet-%i-converged-vs-unconverged',testid));
