;; Copyright (C) 2013-2014, Raytheon BBN Technologies and contributors listed
;; in the AUTHORS file in the MADV distribution's top directory.

;; This file is part of MADV, and is distributed under the terms of the
;; GNU General Public License, with a linking exception, as described in
;; the file LICENSE in the MADV distribution's top directory. 

(defparameter n_iterations 20)

;; Experiment #1:
(dotimes (i n_iterations)
  ;; for a test, change the value of tetralander instrument package
  (compile-and-load "tetralander")
  (startup-propagate)
  (output-current-design-as-vector
   "tetl_expt/tetralander_test1_initial_design.m")
  (marked-values 
   `((INSTRUMENT_PACKAGE_VOLUME 0.01)	; 20x
     (INSTRUMENT_PACKAGE_MASS 5)))	; 10x
  (setf (enabled (find-relation "instrument mass/volume")) nil)
  (setf chart (diagnose-and-repair))
  (output-current-design-as-vector 
   (format nil "tetl_expt/tetralander_test1_~A_design.m" i))
  (let ((header "% Tetrahedral lander, with the following changes:~%~
              % instrument package volume increased 20x, mass increased 10x~%~
              % instrument mass/volume relation disabled~%")
	(filename (format nil "tetralander_test1_~A" i))
	(prefix (format nil "tet_1_~A_" i)))
    (output-diagnose-and-repair-graph
     chart prefix filename "tetl_expt/" header)))

;; Experiment #2:
(dotimes (i n_iterations)
  ;; for a test, change the value of tetralander instrument package
  (compile-and-load "tetralander")
  (startup-propagate)
  (output-current-design-as-vector
   "tetl_expt/tetralander_test2_initial_design.m")
  (marked-values 
   `((INSTRUMENT_PACKAGE_POWER 10)	; 20x
     (INSTRUMENT_PACKAGE_VOLUME 0.0005)	; base case
     (INSTRUMENT_PACKAGE_MASS 0.5)))	; base case
  (setf (enabled (find-relation "instrument mass/volume")) nil)
  (setf chart (diagnose-and-repair))
  (output-current-design-as-vector 
   (format nil "tetl_expt/tetralander_test2_~A_design.m" i))
  (let ((header "% Tetrahedral lander, with the following changes:~%~
              % instrument power demand increased 20x ~%~
              % instrument mass/volume relation disabled~%")
	(filename (format nil "tetralander_test2_~A" i))
	(prefix (format nil "tet_2_~A_" i)))
    (output-diagnose-and-repair-graph
     chart prefix filename "tetl_expt/" header)))


(compile-and-load "tetralander")
(startup-propagate)
(output-current-design-as-vector
 "tetl_expt/tetralander_test2_initial_design.m")
(output-current-design-as-vector
 "tetl_expt/tetralander_test1_initial_design.m")









;; Experiment #3: single difference, same as prior example
(setf use-priorities nil)
(progn
  (setf sim-counts '())
  (setf sim-intervals '(0 1 2 5 10 20 50 100 200 500 1000))
  (dotimes (i (length sim-intervals))
    (setf num-simulations-run 0)
    (setf sim-cache-length (elt sim-intervals i))
    ;; for a test, change the value of tetralander instrument package
    (compile-and-load "tetralander")
    (startup-propagate)
    (output-current-design-as-vector
     "tetl_expt/tetralander_test3_initial_design.m")
    (marked-values 
     `((INSTRUMENT_PACKAGE_POWER 3)
       (INSTRUMENT_PACKAGE_MASS 3)))
    (setf chart (diagnose-and-repair))
    (endpush num-simulations-run sim-counts)
    (output-current-design-as-vector 
     (format nil "tetl_expt/tetralander_test3_~A_design.m" i))
    (let ((header "% Tetrahedral lander, with the following changes:~%~
              % instrument package power increased 6x, mass increased 6x~%")
	  (filename (format nil "tetralander_test3_~A" i))
	  (prefix (format nil "tet_3_~A_" i)))
      (output-diagnose-and-repair-graph
       chart prefix filename "tetl_expt/" header)))
  (with-open-file (out "tetl_expt/tetralander_test3_simcounts.m"
		   :direction :output :if-exists :new-version)
    (list-to-matlab-variable "simintervals" sim-intervals out)
    (list-to-matlab-variable "simcounts" sim-counts out))
  )


;; Experiment #3b: same, but now with automatic duration adjustment
(setf use-priorities nil)
(progn
  (setf sim-counts '())
  (setf sim-intervals '(0))
  (dotimes (i (length sim-intervals))
    (setf num-simulations-run 0)
    (setf sim-cache-length (elt sim-intervals i))
    ;; for a test, change the value of tetralander instrument package
    (compile-and-load "tetralander-no-constraint")
    (startup-propagate)
    (output-current-design-as-vector
     "tetl_expt/tetralander_test3b_initial_design.m")
    (marked-values 
     `((INSTRUMENT_PACKAGE_POWER 3)
       (INSTRUMENT_PACKAGE_MASS 3)))
    (setf chart (diagnose-and-repair))
    (endpush num-simulations-run sim-counts)
    (output-current-design-as-vector 
     (format nil "tetl_expt/tetralander_test3b_~A_design.m" i))
    (let ((header "% Tetrahedral lander, with the following changes:~%~
              % instrument package power increased 6x, mass increased 6x~%")
	  (filename (format nil "tetralander_test3b_~A" i))
	  (prefix (format nil "tet_3b_~A_" i)))
      (output-diagnose-and-repair-graph
       chart prefix filename "tetl_expt/" header)))
  (with-open-file (out "tetl_expt/tetralander_test3b_simcounts.m"
		   :direction :output :if-exists :new-version)
    (list-to-matlab-variable "simintervals" sim-intervals out)
    (list-to-matlab-variable "simcounts" sim-counts out))
  )


;; Experiment #4:
(setf n_iterations 1)
(dotimes (i n_iterations)
  ;; for a test, change the value of tetralander instrument package
  (setf step-size 0.5)
  (setf batch-size 10)
  (compile-and-load "tetralander")
  (startup-propagate)
  (output-current-design-as-vector
   "tetl_expt/tetralander_test4_initial_design.m")
    (marked-values 
     `((INSTRUMENT_PACKAGE_POWER 3)
       (INSTRUMENT_PACKAGE_MASS 3)))
  (setf chart (diagnose-and-repair))
  (output-current-design-as-vector 
   (format nil "tetl_expt/tetralander_test4_~A_design.m" i))
  (let ((header "% Tetrahedral lander, with the following changes:~%~
              % instrument package power increased 6x, mass increased 6x~%")
	(filename (format nil "tetralander_test4_~A" i))
	(prefix (format nil "tet_4_~A_" i)))
    (output-diagnose-and-repair-graph
     chart prefix filename "tetl_expt/" header)))



;; Experiment #5: motor split and rejoin
(setf n_iterations 1)
(dotimes (i n_iterations)
  (setf step-size 0.005)
  (compile-and-load "tetralander")
  (startup-propagate)
  ;; initially expand; motor will need to split
  (output-current-design-as-vector
   "tetl_expt/tetralander_test5_initial_design.m")
    (marked-values 
     `((INSTRUMENT_PACKAGE_POWER 3)
       (INSTRUMENT_PACKAGE_MASS 3)))
  (human-readable-conflict-report (diagnose t))
  (setf chart (run-with-topochange-until-stuck))
  ;; after converging, contract; motor will need to merge
  (marked-values 
   `((BOT_BODY_MASS 0.5)
     (INSTRUMENT_PACKAGE_POWER 0.1)
     (INSTRUMENT_PACKAGE_MASS 0.1)))
  (human-readable-conflict-report (diagnose t))
  (setf chart (merge-charts chart (run-with-topochange-until-stuck)))
  (output-current-design-as-vector 
   (format nil "tetl_expt/tetralander_test5_~A_design.m" i))
  (let ((header "% Tetrahedral lander, motor split and merge.~%")
	(filename (format nil "tetralander_test5_~A" i))
	(prefix (format nil "tet_5_~A_" i)))
    (output-diagnose-and-repair-graph
     chart prefix filename "tetl_expt/" header)))

;; Experiment #6: motor split multiple times
(setf n_iterations 1)
(dotimes (i n_iterations)
  (setf step-size 0.005)
  (compile-and-load "tetralander")
  (startup-propagate)
  ;; initially expand; motor will need to split
  (output-current-design-as-vector
   "tetl_expt/tetralander_test6_initial_design.m")
  (marked-values 
   `((INSTRUMENT_PACKAGE_POWER 10)
     (INSTRUMENT_PACKAGE_MASS 10)))
  (human-readable-conflict-report (diagnose t))
  (setf chart (run-with-topochange-until-stuck 10))
  (output-current-design-as-vector 
   (format nil "tetl_expt/tetralander_test6_~A_design.m" i))
  (let ((header "% Tetrahedral lander, motor split and merge.~%")
	(filename (format nil "tetralander_test6_~A" i))
	(prefix (format nil "tet_6_~A_" i)))
    (output-diagnose-and-repair-graph
     chart prefix filename "tetl_expt/" header)))

;; Experiment #7: split in minidroid
(setf n_iterations 1)
(dotimes (i n_iterations)
  (setf step-size 0.005)
  (compile-and-load "minidroid")
  (startup-propagate)
  (mapcar (ppf mark ?x 'priority 0) (all-elements)) ; speedup
  (output-current-design-as-vector
   "tetl_expt/minidroid_test1_initial_design.m")
  (marked-values
   '((step_height 0.25)))
  (diagnose t)
  (setf chart (run-with-topochange-until-stuck 10))
  (output-current-design-as-vector 
   (format nil "tetl_expt/minidroid_test1_~A_design.m" i))
  (let ((header "% Minidroid, motor split and merge.~%")
	(filename (format nil "minidroid_test1_~A" i))
	(prefix (format nil "mini_1_~A_" i)))
    (output-diagnose-and-repair-graph
     chart prefix filename "tetl_expt/" header)))
