;; Copyright (C) 2013-2014, Raytheon BBN Technologies and contributors listed
;; in the AUTHORS file in the MADV distribution's top directory.

;; This file is part of MADV, and is distributed under the terms of the
;; GNU General Public License, with a linking exception, as described in
;; the file LICENSE in the MADV distribution's top directory. 

(require :xml-rpc)
(use-package :net.xml-rpc)

(defclass simulator-blueprint (modular-relation)
  ((rpccall :initarg :rpc) 
   (simfile :initarg :sim)
   (cachelength :initarg :cache :initform 5)
   (cachevalue :initform 0)
   (cachetimer :initform 0)))

(defun sim-eval (sim-fb risers fallers)
  (declare (ignore risers fallers))
  (with-slots (rpccall simfile cachelength cachevalue cachetimer) sim-fb
    (decf cachetimer)			; count down to zero 1
    (when (<= cachetimer 0)
      (make-python-file)
      (setf cachevalue (call-simulation rpccall simfile))
      (setf cachetimer cachelength))
    cachevalue))
      
(defun make-simulator-blueprint (rpccall simfile risers fallers stressfn)
  (let* ((r-param (mapcar #'find-parameter risers))
	 (f-param (mapcar #'find-parameter fallers)))
    (assert (not (position nil (append r-param f-param))))
    (make-instance 'simulator-blueprint :varsets (list r-param f-param)
		   :risers r-param :fallers f-param 
		   :stress-fn (eval-stress #'sim-eval stressfn)
		   :rpc rpccall :sim simfile)))

;; Generate blueprint files
(defun make-python-file (&optional (filename "parameters.py"))
  (with-open-file (out filename :direction :output :if-exists :new-version)
    ;;(format out "// MADV automatically generated blueprint parameter file~%")
    (dolist (p (remove-if-not (ppf typep ? 'parameter) (all-elements)))
      (when (not (composite? p))
	(python-parameter p out)))))

(defmethod python-parameter ((p parameter) stream)
  (format stream "~A = ~,4F~%" (name p) (get-single-value p)))

(defun file-to-string (name)
  (with-open-file (f name)
    (let ((next-line nil) (all-lines '()))
      (while (setf next-line (read-line f nil))
        (push next-line all-lines))
      (format nil "~{~A~%~}" (reverse all-lines)))))

(defparameter server-addr "http://128.89.77.13:7080/")
(defun call-simulation (rpccall simfile &optional (paramfile "parameters.py"))
  (let* ((sim (file-to-string simfile))
	 (params (file-to-string paramfile))
	 (combined (format nil "~A~%~%~A" params sim)))
    (xml-rpc-call
     (encode-xml-rpc-call rpccall combined)
     :url server-addr)))

;; (call-simulation "flip" "../python/src/example_flip.py") -> -100
;; (call-simulation "straight" "../python/src/example_straight.py") -> 10 [time]
;; (call-simulation "step" "../python/src/example_step.py")
