#!/usr/bin/env python

# SMACH State Machine for the minidroid to climb over a step
# Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
# in the AUTHORS file in the MADV distribution's top directory.
#
# This file is part of MADV, and is distributed under the terms of the
# GNU General Public License, with a linking exception, as described in
# the file LICENSE in the MADV distribution's top directory.


import roslib; roslib.load_manifest('madv_ros_pkg')
import rospy
import smach
import smach_ros
import traceback
import xmlrpclib
import wheels
from math import acos, fabs
from subprocess import call
from threading import Lock

#from smach_tutorials.msg import TestAction, TestGoal
from actionlib import *
from actionlib.msg import *
from std_msgs.msg import Float64
from std_msgs.msg import String
from nav_msgs.msg import Odometry

#import trajectory_msgs.msg

#from pr2_controllers_msgs.msg import JointTrajectoryAction, JointTrajectoryActionGoal
from pr2_controllers_msgs.msg import *
from trajectory_msgs.msg import *

odom_lock = Lock()
angle = [0, 0, 0, 0]
position = [0,0,0]

critical_angle = [0,0,0,0]

metric = 10

time_lock = Lock()
n_seconds = 0

class VelocityState(smach.State):
    def __init__(self, left_vel=0, right_vel=0, num_wheels=1):
        smach.State.__init__(self, outcomes = ['done'])
        self._left_vel = left_vel
        self._right_vel = right_vel
        self._num_wheels = num_wheels
    def execute(self, ud):
        for i in reversed(range(0,self._num_wheels)):
            
            right_pub = rospy.Publisher('/r'+str(i)+'_controller/command', std_msgs.msg.Float64, latch=True)
            left_pub = rospy.Publisher('/l'+str(i)+'_controller/command', std_msgs.msg.Float64, latch=True)
            r = rospy.Rate(10) # 10hz
            if not rospy.is_shutdown():
                if i % 2 == 0:
                    right_pub.publish(self._right_vel)
                    left_pub.publish(self._left_vel)
                else:
                    left_pub.publish(self._left_vel)
                    right_pub.publish(self._right_vel)
                r.sleep()
        return 'done'
        
class EffortState(smach.State):
    def __init__(self, one_eff=0, two_eff=0, three_eff=0):
        smach.State.__init__(self, outcomes = ['done'])
        self._one_eff = one_eff
        self._two_eff = two_eff
        self._three_eff = three_eff
    def execute(self, ud):
        one_pub = rospy.Publisher('/f1_controller/command', std_msgs.msg.Float64, latch=True)
        two_pub = rospy.Publisher('/f2_controller/command', std_msgs.msg.Float64, latch=True)
        three_pub = rospy.Publisher('/f3_controller/command', std_msgs.msg.Float64, latch=True)
        r = rospy.Rate(10) # 10hz
        one_pub.publish(self._one_eff)
        two_pub.publish(self._two_eff)
        three_pub.publish(self._three_eff)
        return 'done'
        

class AngleState(smach.State):
    def __init__(self):
        smach.State.__init__(self, outcomes = ['done'])
        
    def execute(self, ud):
        #Get angle at critical point
        odom_lock.acquire()
        critical_angle = angle
        odom_lock.release()
        global metric 
        metric = fabs(acos( critical_angle[3] )*2 * (180/3.14159265))
        while(metric>90):
            metric  -= 180
        metric = fabs(metric)
        rospy.loginfo(rospy.get_name() + "Orientation %f, %f, %f, %f", critical_angle[0], critical_angle[1], critical_angle[2], critical_angle[3]  )
        rospy.loginfo(rospy.get_name()+"Angle was %f ",metric)

        print("Angle was" + str(metric))
        return 'done'
        
class FinishState(smach.State):
    def __init__(self):
        smach.State.__init__(self, outcomes = ['succeeded', 'aborted'])
        
    def execute(self, ud):        
        time_lock.acquire()
        curr_time = rospy.get_time()
        time_lock.release()
        
        while( curr_time - start_time < 12000):

            time_lock.acquire()
            curr_time = rospy.get_time()
            time_lock.release()

            odom_lock.acquire()
            curr_angle = angle
            odom_lock.release()

            m = acos(curr_angle[3]) * 2 * (180/3.14159265)
            if m > 180:
                m = m - 180

            #print 'angle is '+str(m)

            global end_time
            end_time = 0
            if m > 90:
                # store the end time
                time_lock.acquire()
                end_time = rospy.get_time()
                time_lock.release()
                print 'past 90!'
                return 'succeeded'
        print "Failed to rotate!"
        return 'aborted'



class TransmitGoodState(smach.State):
    def __init__(self):
        smach.State.__init__(self, outcomes = ['succeeded'])
    def execute(self, ud):

        print 'time diff '+str(end_time - start_time)
	try:
		s = xmlrpclib.Server('http://localhost:7080/')
	        #rospy.loginfo(rospy.get_name() + 'good state about to transmit ')
        	s.evaluate('madv', float(end_time - start_time))
	except:
		pass
        #rospy.loginfo(rospy.get_name() + 'good state about to transmit ')
        #rospy.loginfo(rospy.get_name() +str(float(metric))

        #print "good state"

        call(["rosnode", "kill -a"])
        return 'succeeded'
        
  
class TransmitBadState(smach.State):
    def __init__(self):
        smach.State.__init__(self, outcomes = ['aborted'])
    def execute(self, ud):
	try:
		s = xmlrpclib.Server('http://localhost:7080/')
	        #rospy.loginfo(rospy.get_name() + 'good state about to transmit ')
        	s.evaluate('madv', float(-100))
	except:
		pass
    
        print "bad state"
        return 'aborted'
    

def angle_callback(data):
    odom_lock.acquire()
    #rospy.loginfo(rospy.get_name()+"flipper heard %f ",data.pose.pose.orientation.x)
    angle[0] = data.pose.pose.orientation.x
    angle[1] = data.pose.pose.orientation.y
    angle[2] = data.pose.pose.orientation.z
    angle[3] = data.pose.pose.orientation.w
    
    position[0] = data.pose.pose.position.x
    position[1] = data.pose.pose.position.y
    position[2] = data.pose.pose.position.z
    odom_lock.release()

def time_callback(data):
    time_lock.acquire()
    time = data.secs
    time_lock.release()

def main():
    time.sleep(20)
    print "started up!"

    #odom_lock.acquire()
    #curr_angle = angle
    #odom_lock.release()
    #m = acos(curr_angle[3]) * 2 * (180/3.14159265)
    #print 'angle is '+str(m)

    tMsg = JointTrajectoryAction()
    rospy.init_node('smach_example_actionlib')

    rospy.Subscriber("/base_link_pose", Odometry, angle_callback)

    # Create a SMACH state machine
    sm0 = smach.StateMachine(outcomes=['succeeded','aborted','preempted'])



    # Open the container
    with sm0:
        # Add states to the container

        # Add a simple action state. This will use an empty, default goal
        # As seen in TestServer above, an empty goal will always return with
        # GoalStatus.SUCCEEDED, causing this simple action state to return
        # the outcome 'succeeded'
        

        #def traj_result_cb(userdata, status, result):
        #    if status  == GoalStatus.ACTIVE:
             
        def ninety_result_cb(userdata, status, result):
            odom_lock.acquire()
            curr_angle = angle
            odom_lock.release()

            m = acos(curr_angle[3]) * 2 * (180/3.14159265)

            print 'angle is '+str(m)


            return 'succeeded'

        def ninety_plus_result_cb(userdata, status, result):
            odom_lock.acquire()
            curr_angle = angle
            odom_lock.release()

            m = acos(curr_angle[3]) * 2 * (180/3.14159265)

            print 'angle is '+str(m)

            global end_time
            end_time = 0
            if m > 90:
                # store the end time
                time_lock.acquire()
                end_time = rospy.get_time()
                time_lock.release()

                return 'succeeded'
            else:
                return 'aborted'


        
        flipper_goal = JointTrajectoryActionGoal()
        flipper_goal.goal_id.id = 'move_flipper0'
        flipper_goal.goal.trajectory.joint_names = ["base_to_left_flipper","base_to_right_flipper" ]
        flipper_goal.goal.trajectory.points = [JointTrajectoryPoint()]
        
        flipper_goal.goal.trajectory.points[0].positions = [-1.57, -1.57]
        flipper_goal.goal.trajectory.points[0].time_from_start = rospy.Duration(5)
                

        s_goal = JointTrajectoryActionGoal()
        s_goal.goal_id.id = 'move_flipper1'
        s_goal.goal.trajectory.joint_names = ["base_to_left_flipper","base_to_right_flipper" ]
        s_goal.goal.trajectory.points = [JointTrajectoryPoint()]
        
        # s_goal.goal.trajectory.points[0].positions = [-3.14, -3.14]
        s_goal.goal.trajectory.points[0].positions = [-1.7, -1.7]
        s_goal.goal.trajectory.points[0].time_from_start = rospy.Duration(2)


        smach.StateMachine.add('ROTATE', EffortState(one_eff=-500, two_eff=500, three_eff=-8), {'done':'FINISH'})
                               
        
#        smach.StateMachine.add('MOVE_UP',
#                               smach_ros.SimpleActionState('lf_controller/joint_trajectory_action',
#                                                           JointTrajectoryAction,
#                                                           goal=flipper_goal.goal,
#                                                           result_cb=ninety_result_cb),
#                               transitions={'succeeded':'STRAIGHT','aborted':'TransmitBad' })

#        smach.StateMachine.add('STRAIGHT',
#                               smach_ros.SimpleActionState('lf_controller/joint_trajectory_action',
#                                                           JointTrajectoryAction,
#                                                           goal=s_goal.goal,
#                                                           result_cb=ninety_plus_result_cb), {'succeeded':'TransmitGood', 'aborted':'TransmitBad' })
        
        # need failure case? 
#        smach.StateMachine.add('ANGLE', AngleState(), {'done':'TransmitGood'}) 
   
    
#        smach.StateMachine.add('forward', VelocityState(left_vel=2.5, right_vel=2.5, num_wheels=wheels.NUM_WHEELS), {'done':'FINISH'})

        smach.StateMachine.add('FINISH', FinishState(), {'succeeded':'TransmitGood', 'aborted':'TransmitBad'})
        
        smach.StateMachine.add('TransmitGood', TransmitGoodState())
        smach.StateMachine.add('TransmitBad', TransmitBadState())
        
         


    # Create and start the introspection server
    sis = smach_ros.IntrospectionServer('server_name', sm0, '/SM_ROOT')
    sis.start()

    global start_time
    time_lock.acquire()
    start_time = rospy.get_time()
    time_lock.release()

    # Execute SMACH plan 
    outcome = sm0.execute()

    # Wait for ctrl-c to stop the application
    #rospy.spin()
    #sis.stop()

    rospy.signal_shutdown('All done.')


if __name__ == '__main__':
    main()
