#!/usr/bin/env python

# SMACH State Machine for the minidroid to climb over a step
# Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
# in the AUTHORS file in the MADV distribution's top directory.
#
# This file is part of MADV, and is distributed under the terms of the
# GNU General Public License, with a linking exception, as described in
# the file LICENSE in the MADV distribution's top directory.


import roslib; roslib.load_manifest('madv_ros_pkg')
import rospy
import smach
import smach_ros
import traceback
import xmlrpclib
from math import acos, fabs
from subprocess import call
from threading import Lock

#from smach_tutorials.msg import TestAction, TestGoal
from actionlib import *
from actionlib.msg import *
from std_msgs.msg import Float64
from std_msgs.msg import String
from nav_msgs.msg import Odometry
from sensor_msgs.msg import JointState

#import trajectory_msgs.msg

#from pr2_controllers_msgs.msg import JointTrajectoryAction, JointTrajectoryActionGoal
from pr2_controllers_msgs.msg import *
from trajectory_msgs.msg import *

joint_lock = Lock()
joint_pos = [0, 0, 0]

metric = 10

time_lock = Lock()
n_seconds = 0

      
class EffortState(smach.State):
    def __init__(self, one_eff=-10, two_eff=-10, three_eff=-10):
        smach.State.__init__(self, outcomes = ['done'])
        self._one_eff = one_eff
        self._two_eff = two_eff
        self._three_eff = three_eff
    def execute(self, ud):
        one_pub = rospy.Publisher('/f1_controller/command', std_msgs.msg.Float64, latch=True)
        two_pub = rospy.Publisher('/f2_controller/command', std_msgs.msg.Float64, latch=True)
        three_pub = rospy.Publisher('/f3_controller/command', std_msgs.msg.Float64, latch=True)
        r = rospy.Rate(10) # 10hz
        one_pub.publish(self._one_eff)
        two_pub.publish(self._two_eff)
        three_pub.publish(self._three_eff)
        return 'done'
        
        
class FinishState(smach.State):
    def __init__(self):
        smach.State.__init__(self, outcomes = ['succeeded', 'aborted'])
        
    def execute(self, ud):        
        time_lock.acquire()
        curr_time = rospy.get_time()
        time_lock.release()
        
        done = False
        global end_time
        end_time = 0
        crit_angle = -1.91

        while( curr_time - start_time < 60):

            time_lock.acquire()
            curr_time = rospy.get_time()
            time_lock.release()

            joint_lock.acquire()
            curr_pos = joint_pos
            joint_lock.release()

            if (curr_pos[0] < crit_angle):
                if (curr_pos[1] < crit_angle):
                    if (curr_pos[2] < crit_angle):
                        done = True

            if done:
                # store the end time
                time_lock.acquire()
                end_time = rospy.get_time()
                time_lock.release()
                return 'succeeded'
        print "Failed to rotate!"
        return 'aborted'

class TransmitGoodState(smach.State):
    def __init__(self):
        smach.State.__init__(self, outcomes = ['succeeded'])
    def execute(self, ud):

        print 'time diff '+str(end_time - start_time)

        try:
		s = xmlrpclib.Server('http://localhost:7080/')
	        #rospy.loginfo(rospy.get_name() + 'good state about to transmit ')
        	s.evaluate('madv', float(end_time - start_time))
	except:
		pass
        #print "good state"

        call(["rosnode", "kill -a"])
        return 'succeeded'
        
  
class TransmitBadState(smach.State):
    def __init__(self):
        smach.State.__init__(self, outcomes = ['aborted'])
    def execute(self, ud):
        try:
            s = xmlrpclib.Server('http://localhost:7080/')
            #rospy.loginfo(rospy.get_name() + 'good state about to transmit ')
            s.evaluate('madv', float(-100))
        except:
            pass
        print "bad state"
        return 'aborted'

def joint_callback(data):
    joint_lock.acquire()
    #rospy.loginfo(rospy.get_name()+"flipper heard %f ",data.pose.pose.orientation.x)
    
    joint_pos[0] = data.position[data.name.index('base_to_one_flipper')]
    joint_pos[1] = data.position[data.name.index('base_to_two_flipper')]
    joint_pos[2] = data.position[data.name.index('base_to_three_flipper')]

    joint_lock.release()
    

def time_callback(data):
    time_lock.acquire()
    time = data.secs
    time_lock.release()

def main():
    time.sleep(5)
    print "started up!"

    rospy.init_node('smach_example_actionlib')

    rospy.Subscriber("joint_states", JointState, joint_callback)

    # Create a SMACH state machine
    sm0 = smach.StateMachine(outcomes=['succeeded','aborted','preempted'])

    # Open the container
    with sm0:
        # Add states to the container

        # Not sure what to set this effort to?
        smach.StateMachine.add('ROTATE', EffortState(one_eff=-5, two_eff=-5, three_eff=-5), {'done':'FINISH'})
                               
        smach.StateMachine.add('FINISH', FinishState(), {'succeeded':'TransmitGood', 'aborted':'TransmitBad'})
        
        smach.StateMachine.add('TransmitGood', TransmitGoodState())
        smach.StateMachine.add('TransmitBad', TransmitBadState())
        
         


    # Create and start the introspection server
    sis = smach_ros.IntrospectionServer('server_name', sm0, '/SM_ROOT')
    sis.start()

    global start_time
    time_lock.acquire()
    start_time = rospy.get_time()
    time_lock.release()

    # Execute SMACH plan 
    outcome = sm0.execute()

    # Wait for ctrl-c to stop the application
    #rospy.spin()
    #sis.stop()

    rospy.signal_shutdown('All done.')


if __name__ == '__main__':
    main()
