#!/usr/bin/env python

# SMACH State Machine for the minidroid to climb over a step
# Copyright (C) 2011, Raytheon BBN Technologies and contributors listed
# in the AUTHORS file in the MADV distribution's top directory.
#
# This file is part of MADV, and is distributed under the terms of the
# GNU General Public License, with a linking exception, as described in
# the file LICENSE in the MADV distribution's top directory.


import roslib; roslib.load_manifest('madv_ros_pkg')
import rospy
import smach
import smach_ros
import traceback
import xmlrpclib
import wheels
from math import acos, fabs
from subprocess import call
from threading import Lock

#from smach_tutorials.msg import TestAction, TestGoal
from actionlib import *
from actionlib.msg import *
from std_msgs.msg import Float64
from std_msgs.msg import String
from nav_msgs.msg import Odometry

#import trajectory_msgs.msg

#from pr2_controllers_msgs.msg import JointTrajectoryAction, JointTrajectoryActionGoal
from pr2_controllers_msgs.msg import *
from trajectory_msgs.msg import *

odom_lock = Lock()
angle = [0, 0, 0, 0]
position = [0,0,0]

critical_angle = [0,0,0,0]

metric = 10

time_lock = Lock()
n_seconds = 0

      
class EffortState(smach.State):
    def __init__(self, left_eff=0, right_eff=0):
        smach.State.__init__(self, outcomes = ['done'])
        self._left_eff = left_eff
        self._right_eff = -right_eff
    def execute(self, ud):
        right_pub = rospy.Publisher('/rf_controller/command', std_msgs.msg.Float64, latch=True)
        left_pub = rospy.Publisher('/lf_controller/command', std_msgs.msg.Float64, latch=True)
        r = rospy.Rate(10) # 10hz
        right_pub.publish(self._right_eff)
        left_pub.publish(self._left_eff)
        return 'done'
        
        
class FinishState(smach.State):
    def __init__(self):
        smach.State.__init__(self, outcomes = ['succeeded', 'aborted'])
        
    def execute(self, ud):        
        time_lock.acquire()
        curr_time = rospy.get_time()
        time_lock.release()
        
        while( curr_time - start_time < 12000):

            time_lock.acquire()
            curr_time = rospy.get_time()
            time_lock.release()

            odom_lock.acquire()
            curr_angle = angle
            odom_lock.release()

            m = acos(curr_angle[3]) * 2 * (180/3.14159265)
            if m > 180:
                m = m - 180

           # print 'angle is '+str(m)

            global end_time
            end_time = 0
            if m > 90:
                # store the end time
                time_lock.acquire()
                end_time = rospy.get_time()
                time_lock.release()
                print 'past 90!'
                return 'succeeded'
        print "Failed to rotate!"
        return 'aborted'

class TransmitGoodState(smach.State):
    def __init__(self):
        smach.State.__init__(self, outcomes = ['succeeded'])
    def execute(self, ud):

        print 'time diff '+str(end_time - start_time)

        try:
		s = xmlrpclib.Server('http://localhost:7080/')
	        #rospy.loginfo(rospy.get_name() + 'good state about to transmit ')
        	s.evaluate('madv', float(end_time - start_time))
	except:
		pass
        #print "good state"

        call(["rosnode", "kill -a"])
        return 'succeeded'
        
  
class TransmitBadState(smach.State):
    def __init__(self):
        smach.State.__init__(self, outcomes = ['aborted'])
    def execute(self, ud):
        try:
            s = xmlrpclib.Server('http://localhost:7080/')
            #rospy.loginfo(rospy.get_name() + 'good state about to transmit ')
            s.evaluate('madv', float(-100))
        except:
            pass
        print "bad state"
        return 'aborted'
    

def angle_callback(data):
    odom_lock.acquire()
    #rospy.loginfo(rospy.get_name()+"flipper heard %f ",data.pose.pose.orientation.x)
    angle[0] = data.pose.pose.orientation.x
    angle[1] = data.pose.pose.orientation.y
    angle[2] = data.pose.pose.orientation.z
    angle[3] = data.pose.pose.orientation.w
    
    position[0] = data.pose.pose.position.x
    position[1] = data.pose.pose.position.y
    position[2] = data.pose.pose.position.z
    odom_lock.release()

def time_callback(data):
    time_lock.acquire()
    time = data.secs
    time_lock.release()

def main():
    time.sleep(12)
    print "started up!"

    rospy.init_node('smach_example_actionlib')

    rospy.Subscriber("/base_link_pose", Odometry, angle_callback)

    # Create a SMACH state machine
    sm0 = smach.StateMachine(outcomes=['succeeded','aborted','preempted'])

    # Open the container
    with sm0:
        # Add states to the container

        smach.StateMachine.add('ROTATE', EffortState(left_eff=-8, right_eff=8), {'done':'FINISH'})
                               
        smach.StateMachine.add('FINISH', FinishState(), {'succeeded':'TransmitGood', 'aborted':'TransmitBad'})
        
        smach.StateMachine.add('TransmitGood', TransmitGoodState())
        smach.StateMachine.add('TransmitBad', TransmitBadState())
        
         


    # Create and start the introspection server
    sis = smach_ros.IntrospectionServer('server_name', sm0, '/SM_ROOT')
    sis.start()

    global start_time
    time_lock.acquire()
    start_time = rospy.get_time()
    time_lock.release()

    # Execute SMACH plan 
    outcome = sm0.execute()

    # Wait for ctrl-c to stop the application
    #rospy.spin()
    #sis.stop()

    rospy.signal_shutdown('All done.')


if __name__ == '__main__':
    main()
