# XML-RPC server for auto-generating ROS config files and kicking off simulations
 
# Copyright (C) 2011-2013, Raytheon BBN Technologies and contributors listed
# in the AUTHORS file in the MADV distribution's top directory.

# This file is part of MADV, and is distributed under the terms of the
# GNU General Public License, with a linking exception, as described in
# the file LICENSE in the MADV distribution's top directory. 

from twisted.web import xmlrpc, server
from subprocess import call
import time
import os
import subprocess
import multiprocessing
import imp
from threading import Thread

def returnStress(d):
        """
        Data handling function to be added as a callback: handles the
        data by printing the result
        """
        return d

def launchRos():
    subprocess.call('/opt/ros/electric/ros/bin/roslaunch /home/aadler/madv_ros_pkg/madv.launch', shell=True)

def launchRosTest():
    subprocess.call('/opt/ros/electric/ros/bin/roslaunch /home/aadler/madv_ros_pkg/test/madv.launch', shell=True)

def launchMars():
    subprocess.call('/opt/ros/electric/ros/bin/roslaunch /home/aadler/madv_ros_pkg/mars.launch', shell=True)


def createFileRunSim(client, varsContent, simType) :
	#configF = open(config)
	global vars
	#vars = imp.load_source('vars', '', configF)
	vars = imp.new_module('vars')
	exec varsContent in vars.__dict__
	#configData = ConfigParser.ConfigParser()
	#configData.read(config)
	
	global madvPath
	madvPath = '/home/aadler/madv_ros_pkg/'

	# pick one of these:

	if (simType == 1):
		# flip over
		writeAllFiles('test', False, False, int(vars.NUM_WHEELS), int(vars.NUM_WHEEL_SLOTS), vars.END_POS, 'self-right.py') # 4, 33, 0
	elif (simType == 2):
		# drive straight
		writeAllFiles('test', False, True, int(vars.NUM_WHEELS), int(vars.NUM_WHEEL_SLOTS), vars.END_POS, 'go_forward.py') # 4, 33, 5
	elif (simType == 3):
		# step climb
		# writeAllFiles('test', True, True, 33, 33, -0.01905, 'flipper.py')
		writeAllFiles('test', True, True, int(vars.NUM_WHEELS), int(vars.NUM_WHEEL_SLOTS), vars.END_POS, 'flipper.py') # 33, 33, -0.01905, 
	else:
		raise Exception('unknown simulation type')

        print("Wrote configuration files")
        Example.processes[client] = multiprocessing.Process(target = launchRosTest)
        Example.processes[client].start()
        Example.processes[client].join()
        value = Example.values[client]
        Example.deferreds[client].callback(value)
        return

def createMarsRunSim(client, varsContent, simType) :
	#configF = open(config)
	global vars
	#vars = imp.load_source('vars', '', configF)
	vars = imp.new_module('vars')
	exec varsContent in vars.__dict__
	#configData = ConfigParser.ConfigParser()
	#configData.read(config)
	
	global madvPath
	madvPath = '/home/aadler/madv_ros_pkg/'

	# pick one of these:

	if (simType == 1):
		# flat
		writeMarsFiles('')
		writeMarsSpawn('', False)
	elif (simType == 2):
		# flat
		writeMarsFiles('')
		writeMarsSpawn('', True)
	else:
		raise Exception('unknown simulation type')

        print("Wrote configuration files")
        Example.processes[client] = multiprocessing.Process(target = launchMars)
        Example.processes[client].start()
        Example.processes[client].join()
        value = Example.values[client]
        Example.deferreds[client].callback(value)
        return

def writeMarsFiles(path) :
	writeMarsDimensions(path)
	return



def writeAllFiles(path, hasStair, isSingleFlipper, numWheels, numWheelSlots, endPos, behaviorScriptName) :
	writeDimensions(path)
	writeWorld(path, hasStair)
	if hasStair:
		writeStair(path)
	writeControllers(path, True, not(isSingleFlipper), numWheels)
	if isSingleFlipper:
		writeSingleFlipperController(path)
	else:
		writeDualFlipperController(path)
	writeWheelController(path, numWheelSlots, numWheels)
	writeWheels(path, numWheels)
	writeRobot(path)
	writeBotSpawn(path)
	writeLaunchFile(path, behaviorScriptName)
	writeBehaviorParams(numWheels, endPos)
	return




def writeDimensions(path) :
        with open(madvPath+path+'/dimension.urdf.xacro', 'w') as f:
		f.write('<?xml version="1.0" encoding="UTF-8" standalone="no"?>\n')
		f.write('<robot xmlns:body="http://playerstage.sourceforge.net/gazebo/xmlschema/#body" xmlns:controller="http://playerstage.sourceforge.net/gazebo/xmlschema/#controller" xmlns:gazebo="http://playerstage.sourceforge.net/gazebo/xmlschema/#gz" xmlns:geom="http://playerstage.sourceforge.net/gazebo/xmlschema/#geom" xmlns:interface="http://playerstage.sourceforge.net/gazebo/xmlschema/#interface" xmlns:joint="http://playerstage.sourceforge.net/gazebo/xmlschema/#joint" xmlns:model="http://playerstage.sourceforge.net/gazebo/xmlschema/#model" xmlns:physics="http://playerstage.sourceforge.net/gazebo/xmlschema/#physics" xmlns:renderable="http://playerstage.sourceforge.net/gazebo/xmlschema/#renderable" xmlns:rendering="http://playerstage.sourceforge.net/gazebo/xmlschema/#rendering" xmlns:sensor="http://playerstage.sourceforge.net/gazebo/xmlschema/#sensor" xmlns:xacro="http://playerstage.sourceforge.net/gazebo/xmlschema/#interface" xmlns:xi="http://www.w3.org/2001/XInclude" name="minidroid">\n')
		f.write('  <xacro:property name="body_width" value="'+str(vars.BOT_BODY_LENGTH_X)+'"/>\n')
		f.write('  <xacro:property name="body_length" value="'+str(vars.BOT_BODY_LENGTH_Y)+'"/>\n')
		f.write('  <xacro:property name="body_height" value="'+str(vars.BOT_BODY_LENGTH_Z)+'"/>\n')
		f.write('  <xacro:property name="wheel_radius" value="'+str(vars.WHEEL_RADIUS)+'"/>\n')
		f.write('  <xacro:property name="wheel_length" value="'+str(vars.WHEEL_THICKNESS)+'"/>\n')
		f.write('  <xacro:property name="left_flipper_width" value="'+str(vars.FLIPPER_LENGTH_X)+'"/>\n')
		f.write('  <xacro:property name="left_flipper_length" value="'+str(vars.FLIPPER_LENGTH_Y)+'"/>\n')
		f.write('  <xacro:property name="left_flipper_height" value="'+str(vars.FLIPPER_LENGTH_Z)+'"/>\n')
		f.write('  <xacro:property name="right_flipper_width" value="'+str(vars.FLIPPER_LENGTH_X)+'"/>\n')
		f.write('  <xacro:property name="right_flipper_length" value="'+str(vars.FLIPPER_LENGTH_Y)+'"/>\n')
		f.write('  <xacro:property name="right_flipper_height" value="'+str(vars.FLIPPER_LENGTH_Z)+'"/>\n')
		f.write('  <xacro:property name="wheel_buffer" value="0.0025"/>\n')
		f.write('  <xacro:property name="idle_wheel_offset" value="0.01905"/>\n')
		f.write('  <xacro:property name="idle_axle" value="${(-body_length/2) + wheel_radius - idle_wheel_offset}"/>\n')
		f.write('  <xacro:property name="tread_factor" value="'+str(vars.TREAD_FACTOR)+'"/> <!-- was 4.0 / 0.5 -->\n')
		f.write('</robot>\n')
	return

def writeMarsDimensions(path) :
        with open(madvPath+path+'/mars.dimension.urdf.xacro', 'w') as f:
		f.write('<?xml version="1.0" encoding="UTF-8" standalone="no"?>\n')
		f.write('<robot xmlns:body="http://playerstage.sourceforge.net/gazebo/xmlschema/#body" xmlns:controller="http://playerstage.sourceforge.net/gazebo/xmlschema/#controller" xmlns:gazebo="http://playerstage.sourceforge.net/gazebo/xmlschema/#gz" xmlns:geom="http://playerstage.sourceforge.net/gazebo/xmlschema/#geom" xmlns:interface="http://playerstage.sourceforge.net/gazebo/xmlschema/#interface" xmlns:joint="http://playerstage.sourceforge.net/gazebo/xmlschema/#joint" xmlns:model="http://playerstage.sourceforge.net/gazebo/xmlschema/#model" xmlns:physics="http://playerstage.sourceforge.net/gazebo/xmlschema/#physics" xmlns:renderable="http://playerstage.sourceforge.net/gazebo/xmlschema/#renderable" xmlns:rendering="http://playerstage.sourceforge.net/gazebo/xmlschema/#rendering" xmlns:sensor="http://playerstage.sourceforge.net/gazebo/xmlschema/#sensor" xmlns:xacro="http://playerstage.sourceforge.net/gazebo/xmlschema/#interface" xmlns:xi="http://www.w3.org/2001/XInclude" name="marsdroid">\n')
		f.write('  <xacro:property name="side_scale" value="'+str(vars.SIDE_SCALE)+'"/>\n')
		f.write('  <xacro:property name="side_thickness" value="'+str(vars.SIDE_THICKNESS)+'"/>\n')
		f.write('  <xacro:property name="base_mass" value="'+str(vars.BASE_MASS)+'"/>\n')
		f.write('  <xacro:property name="flap_mass" value="'+str(vars.FLAP_MASS)+'"/>\n')
		f.write('  <xacro:property name="max_effort" value="'+str(vars.MAX_EFFORT)+'"/>\n')
		f.write('  <xacro:property name="max_velocity" value="'+str(vars.MAX_VELOCITY)+'"/>\n')
		f.write('</robot>\n')
	return



def writeWorld(path, hasStair) :
	with open(madvPath+path+'/simple_office.world', 'w') as f :
		f.write('<?xml version="1.0"?>\n\n')
		f.write('<gazebo:world \n')
		f.write('  xmlns:xi="http://www.w3.org/2001/XInclude"\n')
		f.write('  xmlns:gazebo="http://playerstage.sourceforge.net/gazebo/xmlschema/#gz" \n')
		f.write('  xmlns:model="http://playerstage.sourceforge.net/gazebo/xmlschema/#model" \n')
		f.write('  xmlns:sensor="http://playerstage.sourceforge.net/gazebo/xmlschema/#sensor" \n')
		f.write('  xmlns:window="http://playerstage.sourceforge.net/gazebo/xmlschema/#window" \n')
		f.write('  xmlns:param="http://playerstage.sourceforge.net/gazebo/xmlschema/#param" \n')
		f.write('  xmlns:body="http://playerstage.sourceforge.net/gazebo/xmlschema/#body" \n')
		f.write('  xmlns:geo="http://willowgarage.com/xmlschema/#geo" \n')
		f.write('  xmlns:geom="http://playerstage.sourceforge.net/gazebo/xmlschema/#geom" \n')
		f.write('  xmlns:joint="http://playerstage.sourceforge.net/gazebo/xmlschema/#joint" \n')
		f.write('  xmlns:interface="http://playerstage.sourceforge.net/gazebo/xmlschema/#interface" \n')
		f.write('  xmlns:ui="http://playerstage.sourceforge.net/gazebo/xmlschema/#ui"\n')
		f.write('  xmlns:rendering="http://playerstage.sourceforge.net/gazebo/xmlschema/#rendering" \n')
		f.write('  xmlns:renderable="http://playerstage.sourceforge.net/gazebo/xmlschema/#renderable" \n')
		f.write('  xmlns:controller="http://playerstage.sourceforge.net/gazebo/xmlschema/#controller"\n')
		f.write('  xmlns:physics="http://playerstage.sourceforge.net/gazebo/xmlschema/#physics" >\n\n')
		f.write('  <verbosity>5</verbosity>\n\n')
		f.write('  <physics:ode>\n')
		f.write('    <stepTime>0.01</stepTime>\n')
		f.write('    <gravity>0 0 -9.8</gravity>\n')
		f.write('    <cfm>1e-7</cfm>\n')
		f.write('    <erp>0.1</erp>\n')
		f.write('        <!--<quickStep>true</quickStep>\n')
		f.write('        <quickStepIters>100</quickStepIters>\n')
		f.write('        <quickStepW>1.3</quickStepW>-->\n')
		f.write('    <contactMaxCorrectingVel>100.0</contactMaxCorrectingVel>\n')
		f.write('    <contactSurfaceLayer>0.001</contactSurfaceLayer>\n')
		f.write('  </physics:ode>\n\n')
		f.write('  <geo:origin>\n')
		f.write('    <lat>37.4270909558</lat><lon>-122.077919338</lon>\n')
		f.write('  </geo:origin>\n\n')
		f.write('  <rendering:gui>\n')
		f.write('    <type>fltk</type>\n')
		f.write('    <size>480 320</size>\n')
		f.write('    <pos>0 0</pos>\n')
		f.write('    <frames>\n')
		f.write('      <row height="100%">\n')
		f.write('        <camera width="100%">\n')
		f.write('          <xyz>0 0 1</xyz>\n')
		f.write('          <rpy>0 90 90</rpy>\n')
		f.write('        </camera>\n')
		f.write('      </row>\n')
		f.write('    </frames>\n')
		f.write('  </rendering:gui>\n\n')
		f.write('  <rendering:ogre>\n')
		f.write('    <ambient>1.0 1.0 1.0 1.0</ambient>\n')
		f.write('    <sky>\n')
		f.write('      <material>Gazebo/CloudySky</material>\n')
		f.write('    </sky>\n')
		f.write('    <grid>false</grid>\n')
		f.write('    <maxUpdateRate>10</maxUpdateRate>\n')
		f.write('  </rendering:ogre>\n\n')
		f.write('  <model:physical name="gplane">\n')
		f.write('    <xyz>0 0 0</xyz>\n')
		f.write('    <rpy>0 0 0</rpy>\n')
		f.write('    <static>true</static>\n\n')
		f.write('    <body:plane name="plane1_body">\n')
		f.write('      <geom:plane name="plane1_geom">\n')
		f.write('        <normal>0 0 1</normal>\n')
		f.write('        <size>2000 2000</size>\n')
		f.write('        <segments>10 10</segments>\n')
		f.write('        <uvTile>1000 1000</uvTile>\n')
		f.write('        <material>Gazebo/GrassFloor</material>\n')
		f.write('      </geom:plane>\n')
		f.write('    </body:plane>\n')
		f.write('  </model:physical>\n\n')
		if hasStair:
			f.write('  <model:physical name="stair">\n')
			f.write('    <include embedded="true">\n')
			f.write('      <xi:include href="stair.model" />\n')
			f.write('    </include>\n')
			f.write('  </model:physical>\n\n')
		f.write('</gazebo:world>\n')
	return

def writeStair(path) :
	with open(madvPath+path+'/stair.model', 'w') as f :
		f.write('<?xml version="1.0" encoding="UTF-8" standalone="no"?>\n')
		f.write('<model:physical xmlns:model="http://playerstage.sourceforge.net/gazebo/xmlschema/#model" xmlns:body="http://playerstage.sourceforge.net/gazebo/xmlschema/#body" xmlns:controller="http://playerstage.sourceforge.net/gazebo/xmlschema/#controller" xmlns:gazebo="http://playerstage.sourceforge.net/gazebo/xmlschema/#gz" xmlns:geo="http://willowgarage.com/xmlschema/#geo" xmlns:geom="http://playerstage.sourceforge.net/gazebo/xmlschema/#geom" xmlns:interface="http://playerstage.sourceforge.net/gazebo/xmlschema/#interface" xmlns:joint="http://playerstage.sourceforge.net/gazebo/xmlschema/#joint" xmlns:param="http://playerstage.sourceforge.net/gazebo/xmlschema/#param" xmlns:physics="http://playerstage.sourceforge.net/gazebo/xmlschema/#physics" xmlns:renderable="http://playerstage.sourceforge.net/gazebo/xmlschema/#renderable" xmlns:rendering="http://playerstage.sourceforge.net/gazebo/xmlschema/#rendering" xmlns:sensor="http://playerstage.sourceforge.net/gazebo/xmlschema/#sensor" xmlns:ui="http://playerstage.sourceforge.net/gazebo/xmlschema/#ui" xmlns:window="http://playerstage.sourceforge.net/gazebo/xmlschema/#window" xmlns:xi="http://www.w3.org/2001/XInclude" name="stair">\n')
		f.write('  <xyz>'+str(vars.STEP_POSITION_X)+' '+str(vars.STEP_POSITION_Y)+' '+str(vars.STEP_POSITION_Z)+'</xyz>\n')
		f.write('  <rpy>0.0 0.0 0.0</rpy>\n')
		f.write('  <static>true</static>\n')
		f.write('  <body:box name="stair">\n')
		f.write('    <geom:box name="wall_1_geom">\n')
		f.write('      <mesh>default</mesh>\n')
		f.write('      <size>'+str(vars.STEP_LENGTH_X)+' '+str(vars.STEP_LENGTH_Y)+' '+str(vars.STEP_LENGTH_Z)+'</size>\n')
		f.write('      <visual>\n')
		f.write('        <size>'+str(vars.STEP_LENGTH_X)+' '+str(vars.STEP_LENGTH_Y)+' '+str(vars.STEP_LENGTH_Z)+'</size>\n')
		f.write('        <material>Gazebo/WoodFloor</material>\n')
		f.write('        <mesh>unit_box</mesh>\n')
		f.write('      </visual>\n')
		f.write('    </geom:box>\n')
		f.write('  </body:box>\n')
		f.write('</model:physical>\n')
	return



def writeControllers(path, leftFlip, rightFlip, numWheelControllers) :
	with open(madvPath+path+'/controllers.launch', 'w') as f :
		f.write('<?xml version="1.0" encoding="UTF-8" standalone="no"?>\n')
		f.write('<launch>\n')
		f.write('  <!--Start controllers-->\n')
		f.write('  <node args="')

		controllers = ''
		# now we need to figure out which controllers we need:
		if leftFlip:
			controllers += 'lf_controller '
		if rightFlip:
			controllers += 'rf_controller '
		for i in range(numWheelControllers):
			controllers += 'r'+str(i)+'_controller l'+str(i)+'_controller '
		f.write(controllers.strip())
		f.write('" name="default_controllers_spawner" output="screen" pkg="pr2_controller_manager" type="spawner"/>\n')
		f.write('</launch>\n')
	return

def writeSingleFlipperController(path) :
	with open(madvPath+path+'/madv_flipper_controller.yaml', 'w') as f :
		f.write('lf_controller:\n')
		f.write('  type: robot_mechanism_controllers/JointTrajectoryActionController\n')
		f.write('  joints:\n')
		f.write('    - base_to_left_flipper\n')
		f.write('    - base_to_right_flipper\n')
		f.write('  gains:\n')
		f.write('    base_to_left_flipper: {p: 200, d: 10}\n')
		f.write('    base_to_right_flipper: {p: 200, d: 10}\n')
		f.write('  joint_trajectory_action_node:\n')
		f.write('    joints:\n')
		f.write('      - base_to_left_flipper\n')
		f.write('      - base_to_right_flipper\n')
		f.write('    constraints:\n')
		f.write('      goal_time: 0\n')
		f.write('      base_to_left_flipper:\n')
		f.write('        goal: 0.1\n')
		f.write('      base_to_right_flipper:\n')
		f.write('        goal: 0.1\n')
	return

def writeDualFlipperController(path) :
	with open(madvPath+path+'/madv_flipper_controller.yaml', 'w') as f :
		f.write('lf_controller:\n')
		f.write('  type: robot_mechanism_controllers/JointEffortController\n')
		f.write('  joint: base_to_left_flipper\n\n')
		f.write('rf_controller:\n')
		f.write('  type: robot_mechanism_controllers/JointEffortController\n')
		f.write('  joint: base_to_right_flipper\n')
	return

def writeWheelController(path, numWheelSlots,numControllers) :
	with open(madvPath+path+'/madv_wheel_controller.yaml', 'w') as f :
		f.write('num_wheels : '+str(numWheelSlots)+'\n\n')
		for i in range(numControllers):
			f.write('r'+str(i)+'_controller:\n')
			f.write('   type: robot_mechanism_controllers/JointVelocityController\n')
			f.write('   joint: base_to_right_'+str(i)+'_wheel\n')
			f.write('   pid:\n')
			f.write('     p: 1.0\n')
			f.write('     i: 20.0\n')
			f.write('     i_clamp: 100.0\n\n')
			f.write('l'+str(i)+'_controller:\n')
			f.write('   type: robot_mechanism_controllers/JointVelocityController\n')
			f.write('   joint: base_to_left_'+str(i)+'_wheel\n')
			f.write('   pid:\n')
			f.write('     p: 1.0\n')
			f.write('     i: 20.0\n')
			f.write('     i_clamp: 100.0\n\n')
	return

def writeWheels(path, numWheels) :
	with open(madvPath+path+'/wheels.urdf.xacro', 'w') as f :
		f.write('<?xml version="1.0" encoding="UTF-8" standalone="no"?>\n')
		f.write('<robot xmlns:body="http://playerstage.sourceforge.net/gazebo/xmlschema/#body" xmlns:controller="http://playerstage.sourceforge.net/gazebo/xmlschema/#controller" xmlns:gazebo="http://playerstage.sourceforge.net/gazebo/xmlschema/#gz" xmlns:geom="http://playerstage.sourceforge.net/gazebo/xmlschema/#geom" xmlns:interface="http://playerstage.sourceforge.net/gazebo/xmlschema/#interface" xmlns:joint="http://playerstage.sourceforge.net/gazebo/xmlschema/#joint" xmlns:model="http://playerstage.sourceforge.net/gazebo/xmlschema/#model" xmlns:physics="http://playerstage.sourceforge.net/gazebo/xmlschema/#physics" xmlns:renderable="http://playerstage.sourceforge.net/gazebo/xmlschema/#renderable" xmlns:rendering="http://playerstage.sourceforge.net/gazebo/xmlschema/#rendering" xmlns:sensor="http://playerstage.sourceforge.net/gazebo/xmlschema/#sensor" xmlns:xacro="http://playerstage.sourceforge.net/gazebo/xmlschema/#interface" xmlns:xi="http://www.w3.org/2001/XInclude" name="minidroid">\n')
		for i in range(numWheels):
			f.write('  <xacro:wheel_axle wheel_index="'+str(i)+'"/>\n')
		f.write('</robot>\n')
	return

def writeRobot(path) :
	with open(madvPath+path+'/madv.urdf.xacro', 'w') as f :
		f.write('<?xml version="1.0"?>\n')
		f.write('<robot \n')
		f.write('       xmlns:xi="http://www.w3.org/2001/XInclude"\n')
		f.write('       xmlns:gazebo="http://playerstage.sourceforge.net/gazebo/xmlschema/#gz"\n')
		f.write('       xmlns:model="http://playerstage.sourceforge.net/gazebo/xmlschema/#model"\n')
		f.write('       xmlns:sensor="http://playerstage.sourceforge.net/gazebo/xmlschema/#sensor"\n')
		f.write('       xmlns:body="http://playerstage.sourceforge.net/gazebo/xmlschema/#body"\n')
		f.write('       xmlns:geom="http://playerstage.sourceforge.net/gazebo/xmlschema/#geom"\n')
		f.write('       xmlns:joint="http://playerstage.sourceforge.net/gazebo/xmlschema/#joint"\n')
		f.write('       xmlns:interface="http://playerstage.sourceforge.net/gazebo/xmlschema/#interface"\n')
		f.write('       xmlns:rendering="http://playerstage.sourceforge.net/gazebo/xmlschema/#rendering"\n')
		f.write('       xmlns:renderable="http://playerstage.sourceforge.net/gazebo/xmlschema/#renderable"\n')
		f.write('       xmlns:controller="http://playerstage.sourceforge.net/gazebo/xmlschema/#controller"\n')
		f.write('       xmlns:physics="http://playerstage.sourceforge.net/gazebo/xmlschema/#physics"\n')
		f.write('       xmlns:xacro="http://playerstage.sourceforge.net/gazebo/xmlschema/#interface" name="minidroid">\n\n')
		f.write('  <include filename="$(find madv_ros_pkg)/'+str(path)+'/dimension.urdf.xacro" />\n')
		f.write('\n')
		f.write('  <!-- Parameters for robot dimensions -->\n')
		f.write('\n')
		f.write('  <xacro:property name="pi" value="3.1415" />\n')
		f.write('\n')
		f.write('\n')
		f.write('  <!-- Macro for assigning identity matrix as moment of intertia -->\n')
		f.write('  <xacro:macro name="default_inertial" params="mass">\n')
		f.write('    <inertial>\n')
		f.write('      <mass value="${mass}" />\n')
		f.write('      <inertia ixx="${mass}" ixy="0.0" ixz="0.0"\n')
		f.write('               iyy="${mass}" iyz="0.0"\n')
		f.write('               izz="${mass}" />\n')
		f.write('    </inertial>\n')
		f.write('  </xacro:macro>\n')
		f.write('\n')
		f.write('  <!-- Body -->\n')
		f.write('  <link name="base_link">\n')
		f.write('    <visual>\n')
		f.write('      <geometry>\n')
		f.write('        <box size="${body_width} ${body_length} ${body_height}"/>\n')
		f.write('      </geometry>\n')
		f.write('      <material name="blue">\n')
		f.write('        <color rgba="0 0 .8 1"/>\n')
		f.write('      </material>\n')
		f.write('    </visual>\n')
		f.write('    <collision>\n')
		f.write('      <geometry>\n')
		f.write('        <box size="${body_width} ${body_length} ${body_height}"/>\n')
		f.write('      </geometry>\n')
		f.write('    </collision>\n')
		f.write('    <xacro:default_inertial mass="1.15"/>\n')
		f.write('  </link>\n')
		f.write('\n')
		f.write('\n')
		f.write('  <!-- Macro for placing wheels on robot, includes wheel and joint -->\n')
		f.write('  <xacro:macro name="wheel" params="prefix reflect position">\n')
		f.write('    <link name="${prefix}_wheel">\n')
		f.write('      <visual>\n')
		f.write('	<geometry>\n')
		f.write('	  <cylinder radius="${wheel_radius}" length="${wheel_length}" />\n')
		f.write('	</geometry>\n')
		f.write('	<origin xyz="0 0 0" rpy="0 ${pi/2} 0"/>\n')
		f.write('	<material name="white">\n')
		f.write('	  <color rgba="1 1 1 1"/>\n')
		f.write('	</material>\n')
		f.write('      </visual>\n')
		f.write('      <collision>\n')
		f.write('	<geometry>\n')
		f.write('	  <cylinder radius="${wheel_radius}" length="${wheel_length}" />\n')
		f.write('	</geometry>  \n')
		f.write('	<origin xyz="0 0 0" rpy="0 ${pi/2} 0"/>\n')
		f.write('	<!--<contact_coefficients mu="0.999" kp="0.01" kd="5.0e+10"/>--> \n')
		f.write('      </collision>\n')
		f.write('      <xacro:default_inertial mass="0.05"/>\n')
		f.write('    </link>\n')
		f.write('\n')
		f.write('    <gazebo reference="${prefix}_wheel">\n')
		f.write('      <mu1 value="0.999*2" />\n')
		f.write('      <mu2 value="0.999*2" />\n')
		f.write('      <kp  value="1.0e+2"/>\n')
		f.write('      <kd  value="5.0e+10"  />\n')
		f.write('      <material>PR2/Grey2</material>\n')
		f.write('    </gazebo>\n')
		f.write('\n')
		f.write('\n')
		f.write('    <joint name="base_to_${prefix}_wheel" type="continuous">\n')
		f.write('      <axis xyz="1 0 0" />\n')
		f.write('      <parent link="base_link"/>\n')
		f.write('      <child link="${prefix}_wheel"/>\n')
		f.write('      <origin xyz="${reflect*(wheel_length+body_width+wheel_buffer)/2} ${position} 0" />\n')
		f.write('      <limit effort="'+str(vars.WHEEL_EFFORT_LIMIT)+'" velocity="'+str(vars.WHEEL_VELOCITY_LIMIT)+'" />\n')
		f.write('      <safety_controller k_velocity="1" />\n')
		f.write('    </joint>\n')
		f.write('\n')
		f.write('    <transmission type="pr2_mechanism_model/SimpleTransmission" name="${prefix}_wheel_trans">\n')
		f.write('      <actuator name="${prefix}_wheel_motor" />\n')
		f.write('      <joint name="base_to_${prefix}_wheel" />\n')
		f.write('      <mechanicalReduction>1</mechanicalReduction>\n')
		f.write('      <motorTorqueConstant>1</motorTorqueConstant>\n')
		f.write('    </transmission>\n')
		f.write('  </xacro:macro>\n')
		f.write('\n')
		f.write('  <!-- Macro for placing flippers on robot, includes joints -->\n')
		f.write('  <xacro:macro name="flipper" params="prefix reflect width length height">\n')
		f.write('    <link name="${prefix}_flipper">\n')
		f.write('      <visual>\n')
		f.write('	<geometry>\n')
		f.write('	  <box size="${width} ${length} ${height}" />\n')
		f.write('	</geometry>\n')
		f.write('	<origin xyz="0 ${length/2} 0" rpy="0 0 0"/> \n')
		f.write('	<material name="grey">\n')
		f.write('	  <color rgba="0.5 0.5 0.5 1"/>\n')
		f.write('	</material>\n')
		f.write('      </visual>\n')
		f.write('\n')
		f.write('      <collision>\n')
		f.write('	<geometry>\n')
		f.write('	  <box size="${width} ${length} ${height}" />\n')
		f.write('	</geometry>  \n')
		f.write('	<origin xyz="0  ${length/2} 0" rpy="0 0 0"/>\n')
		f.write('	<!--<contact_coefficients mu="0.0000001" kp="0.01" kd="0.01"/>\n')
		f.write('	<surface>  \n')
		f.write('	  <ode mu="0.999" mu2="0.999"/>\n')
		f.write('	  </surface>-->\n')
		f.write('      </collision>\n')
		f.write('      <xacro:default_inertial mass="0.1"/>\n')
		f.write('    </link>\n')
		f.write('\n')
		f.write('\n')
		f.write('    <gazebo reference="${prefix}_flipper">\n')
		f.write('      <mu1 value="0.001" />\n')
		f.write('      <mu2 value="0.001" />\n')
		f.write('      <kp  value="1.0e+2"/>\n')
		f.write('      <kd  value="5.0e+10"  />\n')
		f.write('    </gazebo>\n')
		f.write('\n')
		f.write('    <joint name="base_to_${prefix}_flipper" type="continuous">\n')
		f.write('      <origin xyz="${reflect*(body_width/2 + wheel_length + wheel_buffer + width/2)} ${(-body_length/2) +wheel_radius - idle_wheel_offset} 0" rpy="0 0 0"/>\n')
		f.write('      <axis xyz="1 0 0" />\n')
		f.write('      <parent link="base_link"/>\n')
		f.write('      <child link="${prefix}_flipper"/>\n')
		f.write('      <safety_controller k_velocity="10"/> -->\n')
		f.write('      <limit effort="'+str(vars.FLIPPER_EFFORT_LIMIT)+'" velocity="'+str(vars.FLIPPER_VELOCITY_LIMIT)+'" />\n')
		f.write('      <safety_controller k_velocity="1"/>\n')
		f.write('    </joint> \n')
		f.write('\n')
		f.write('  <transmission type="pr2_mechanism_model/SimpleTransmission" name="${prefix}_flipper_trans">\n')
		f.write('    <actuator name="${prefix}_flipper_motor" />\n')
		f.write('    <joint name="base_to_${prefix}_flipper" />\n')
		f.write('    <mechanicalReduction>1</mechanicalReduction>\n')
		f.write('    <motorTorqueConstant>1</motorTorqueConstant>\n')
		f.write('  </transmission>\n')
		f.write('\n')
		f.write('  </xacro:macro>\n')
		f.write('\n')
		f.write('\n')
		f.write('  <xacro:flipper prefix="left" reflect="1" width="${left_flipper_width}" length="${left_flipper_length}" height="${left_flipper_height}" />\n')
		f.write('  <xacro:flipper prefix="right" reflect="-1" width="${right_flipper_width}" length="${right_flipper_length}" height="${right_flipper_height}" />\n')
		f.write('\n')
		f.write('  <xacro:macro name="wheel_axle" params="wheel_index">\n')
		f.write('    <xacro:wheel prefix="left_${wheel_index}" reflect="1" position="${(-body_length/2+idle_wheel_offset) + wheel_radius * tread_factor * wheel_index}" />\n')
		f.write('    <xacro:wheel prefix="right_${wheel_index}" reflect="-1" position="${(-body_length/2+idle_wheel_offset)+ wheel_radius * tread_factor * wheel_index}" />\n')
		f.write('  </xacro:macro>\n')
		f.write('\n')
		f.write('  <include filename="$(find madv_ros_pkg)/'+path+'/wheels.urdf.xacro" />\n')
		f.write('\n')
		f.write('<!--  <xacro:wheel prefix="left_2" reflect="1" position="${idle_axle}" /> -->\n')
		f.write('<!--  <xacro:wheel prefix="right_2" reflect="-1" position="${idle_axle}"/> -->\n')
		f.write('\n')
		f.write(' <gazebo reference="left_flipper">\n')
		f.write('    <material>PR2/Grey1</material>\n')
		f.write('  </gazebo>\n')
		f.write('\n')
		f.write(' <gazebo reference="right_flipper">\n')
		f.write('    <material>PR2/Grey1</material>\n')
		f.write('  </gazebo>\n')
		f.write('\n')
		f.write(' <gazebo reference="base_link">\n')
		f.write('    <material>Gazebo/Grey</material>\n')
		f.write('  </gazebo>\n')
		f.write('\n')
		f.write('  <gazebo>\n')
		f.write('      <!-- PR2_ACTARRAY -->\n')
		f.write('      <controller:gazebo_ros_controller_manager name="gazebo_ros_controller_manager" plugin="libgazebo_ros_controller_manager.so">\n')
		f.write('        <alwaysOn>true</alwaysOn>\n')
		f.write('        <updateRate>1000.0</updateRate>\n')
		f.write('      </controller:gazebo_ros_controller_manager>\n')
		f.write('\n')
		f.write('      <!-- gazebo_ros_p3d for position groundtruth -->\n')
		f.write('      <controller:gazebo_ros_p3d name="p3d_link1_controller" plugin="libgazebo_ros_p3d.so">\n')
		f.write('          <alwaysOn>true</alwaysOn>\n')
		f.write('          <updateRate>1000.0</updateRate>\n')
		f.write('          <bodyName>base_link</bodyName>\n')
		f.write('          <topicName>base_link_pose</topicName>\n')
		f.write('          <frameName>map</frameName>\n')
		f.write('          <interface:position name="p3d_link1_position"/>\n')
		f.write('      </controller:gazebo_ros_p3d>\n')
		f.write('  </gazebo>\n')
		f.write('\n')
		f.write('\n')
		f.write('\n')
		f.write('</robot>\n')
	return

def writeBotSpawn(path) :
	with open(madvPath+path+'/bot_spawn.launch', 'w') as f :
		f.write('<?xml version="1.0" encoding="UTF-8" standalone="no"?>\n')
		f.write('<launch>\n')
		f.write('  <node args="-urdf -param robot_description -y '+str(vars.BOT_POSITION_Y)+' -z '+str(vars.BOT_POSITION_Z)+' -model minidroid" name="spawn_minidroid" pkg="gazebo" respawn="false" type="spawn_model"/>\n')
		f.write('</launch>\n')
	return

def writeMarsSpawn(path, rotate) :
	with open(madvPath+path+'/mars_spawn.launch', 'w') as f :
		f.write('<?xml version="1.0" encoding="UTF-8" standalone="no"?>\n')
		f.write('<launch>\n')
		f.write('  <node args="-urdf -param robot_description -x 0 -y 0 -z '+str(vars.MARS_POSITION_Z))
		if rotate:
			f.write(' -R 1.91')
		f.write(' -model marsdroid" name="spawn_marsdroid" pkg="gazebo" respawn="false" type="spawn_model"/>\n')
		f.write('</launch>\n')
	return

def writeLaunchFile(path, behaviorScriptName) :
	with open(madvPath+path+'/madv.launch', 'w') as f :
		f.write('<launch>\n')
		f.write('  <arg name="gui" default="False" />\n')
		f.write('  <param name="use_gui" value="$(arg gui)"/>\n')
		#f.write('  <!-- <node name="rxconsole" pkg="rxtools" type="rxconsole" />-->\n')
		f.write('  <!-- load world in gazebo -->\n')
		f.write('  <!-- add -r as an arg to run headleass -->\n')
		f.write('  <node name="gazebo" pkg="gazebo" type="gazebo" args="$(find madv_ros_pkg)/'+path+'/simple_office.world" respawn="false" output="screen"/>\n\n')
		f.write('  <!-- parse robot file from xacro file -->\n')
		f.write('  <param name="robot_description" command="$(find xacro)/xacro.py \'$(find madv_ros_pkg)/'+path+'/madv.urdf.xacro\'" />\n\n')
		f.write('  <!-- load robot model into gazebo -->\n')
		f.write('  <include file="$(find madv_ros_pkg)/'+path+'/bot_spawn.launch" />\n')
		f.write('  <!-- load robot controller file -->\n')
		f.write('  <rosparam file="$(find madv_ros_pkg)/'+path+'/madv_flipper_controller.yaml" command="load"/>\n')
		f.write('  <rosparam file="$(find madv_ros_pkg)/'+path+'/madv_wheel_controller.yaml" command="load"/>\n\n')
		f.write('  <!-- Controller Manager -->\n')
		f.write('  <include file="$(find pr2_controller_manager)/controller_manager.launch" /> \n')
		f.write('  <include file="$(find madv_ros_pkg)/'+path+'/controllers.launch" />\n\n')
		f.write('  <node name="flipper" pkg="madv_ros_pkg" type="'+behaviorScriptName+'" required="true"/>\n\n')
		f.write('</launch>\n')
	return

def writeBehaviorParams(numWheels, endPos) :
	with open(madvPath+'wheels.py', 'w') as f :
		f.write('NUM_WHEELS = '+str(numWheels)+'\n')
		f.write('DRIVE_END_POS = '+str(endPos)+'\n')
		f.write('STEP_END_POS = '+str(endPos)+'\n')
		f.write('STEP_NUM_WHEELS = '+str(numWheels)+'\n')
	return

def runSim(client, dimensionsContent, wheelsContent, wheelControllerContent, controllersContent, stairContent, numWheelsContent, botSpawnContent):
        print("Writing out ROS configuration files for client " + client)
        
        #path = '../../../java/resources/xmlModels/'
        #path = '$MADV_ROS_PKG/'
        path = '/home/aadler/madv_ros_pkg/'
        f = open(path+'dimension.urdf.xacro', 'w')
        f.write(dimensionsContent);
        f.close()
        
        f = open(path+'wheels.urdf.xacro', 'w')
        f.write(wheelsContent);
        f.close()
        
        f = open(path+'madv_wheel_controller.yaml', 'w')
        f.write(wheelControllerContent);
        f.close()
        
        f = open(path+'controllers.launch', 'w')
        f.write(controllersContent);
        f.close()
        
        f = open(path+'stair.model', 'w')
        f.write(stairContent);
        f.close()
        
        f = open(path+'wheels.py', 'w')
        f.write(numWheelsContent);
        f.close()
        
                
        f = open(path+'bot_spawn.launch', 'w')
        f.write(botSpawnContent);
        f.close()
        
        print("Wrote configuration files")
        Example.processes[client] = multiprocessing.Process(target = launchRos)
        Example.processes[client].start()
        Example.processes[client].join()
        value = Example.values[client]
        Example.deferreds[client].callback(value)
        return

def sendValue(self, rosThread, d, value):
    rosThread.join()
    d.callback(float(value))
    return



class Example(xmlrpc.XMLRPC):
    """
    An example object to be published.
    """
    deferreds = {}
    processes = {}
    experiments = {}
    values = {}

        
    def xmlrpc_evaluate(self, client, value):
        Example.values[client] = value
        return 'DONE'

    def xmlrpc_flip(self, configContent):
        client = 'madv'
        d = defer.Deferred()
        d.addCallback(returnStress)
        Example.deferreds[client] = d
        sim=Thread(target = createFileRunSim, args=(client, configContent, 1))
        #sim = multiprocessing.Process(target = runSim, args=(client, dimensionsContent, wheelsContent, wheelControllerContent, controllersContent, stairContent, numWheelsContent, botSpawnContent))
        sim.start()
        #p.join()
        return d

    def xmlrpc_straight(self, configContent):
        client = 'madv'
        d = defer.Deferred()
        d.addCallback(returnStress)
        Example.deferreds[client] = d
        sim=Thread(target = createFileRunSim, args=(client, configContent, 2))
        #sim = multiprocessing.Process(target = runSim, args=(client, dimensionsContent, wheelsContent, wheelControllerContent, controllersContent, stairContent, numWheelsContent, botSpawnContent))
        sim.start()
        #p.join()
        return d

    def xmlrpc_step(self, configContent):
        client = 'madv'
        d = defer.Deferred()
        d.addCallback(returnStress)
        Example.deferreds[client] = d
        sim=Thread(target = createFileRunSim, args=(client, configContent, 3))
        #sim = multiprocessing.Process(target = runSim, args=(client, dimensionsContent, wheelsContent, wheelControllerContent, controllersContent, stairContent, numWheelsContent, botSpawnContent))
        sim.start()
        #p.join()
        return d

    def xmlrpc_mars_flat(self, configContent):
        client = 'madv'
        d = defer.Deferred()
        d.addCallback(returnStress)
        Example.deferreds[client] = d
        sim=Thread(target = createMarsRunSim, args=(client, configContent, 1))
        #sim = multiprocessing.Process(target = runSim, args=(client, dimensionsContent, wheelsContent, wheelControllerContent, controllersContent, stairContent, numWheelsContent, botSpawnContent))
        sim.start()
        #p.join()
        return d

    def xmlrpc_mars_rotate(self, configContent):
        client = 'madv'
        d = defer.Deferred()
        d.addCallback(returnStress)
        Example.deferreds[client] = d
        sim=Thread(target = createMarsRunSim, args=(client, configContent, 2))
        #sim = multiprocessing.Process(target = runSim, args=(client, dimensionsContent, wheelsContent, wheelControllerContent, controllersContent, stairContent, numWheelsContent, botSpawnContent))
        sim.start()
        #p.join()
        return d

    # need one xmlrpc call per simulation
    # need to change creteFileRunSim so that it get the content not the filename for the config data
    # need to create a client to test this

if __name__ == '__main__':
	#createFileRunSim('foo','example.py') # example.py / example_step.py

	from twisted.internet import *
	r = Example()
	print( "Starting twistd reactor" )
	reactor.listenTCP(7080, server.Site(r))
	reactor.run()
